from django.conf import settings
from django.core import urlresolvers
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.datastructures import MultiValueDictKeyError
from django.utils.safestring import mark_safe
try:
    from django.utils.simplejson.encoder import JSONEncoder
except ImportError:
    from simplejson.encoder import JSONEncoder
from django.utils.translation import ugettext as _
from satchmo_store.contact.models import Contact
from satchmo_store.shop.signals import order_success
from product.models import Product
from product.views import find_product_template
from satchmo_store.shop.exceptions import CartAddProhibited
from satchmo_store.shop.models import Cart, Order
from satchmo_store.shop.signals import satchmo_cart_changed
from satchmo_store.shop.views.cart import product_from_post
from satchmo_utils.views import bad_or_missing
from satchmo_ext.wishlist.models import ProductWish
import logging

log = logging.getLogger('wishlist.view')

def wishlist_view(request, message=""):
    try:
        contact = Contact.objects.from_request(request)
    except Contact.DoesNotExist:
        return _wishlist_requires_login(request)
        
    wishes = ProductWish.objects.filter(contact=contact)

    ctx = RequestContext(request, {
        'wishlist' : wishes,
        'wishlist_message' : message,
    })

    return render_to_response('wishlist/index.html', context_instance=ctx)

def wishlist_add(request):
    """Add an item to the wishlist."""
    try:
        contact = Contact.objects.from_request(request)
    except Contact.DoesNotExist:
        return _wishlist_requires_login(request)
        
    log.debug('FORM: %s', request.POST)
    formdata = request.POST.copy()
    productslug = None
    if formdata.has_key('productname'):
        productslug = formdata['productname']
    try:
        product, details = product_from_post(productslug, formdata)
        template = find_product_template(product)
        
    except (Product.DoesNotExist, MultiValueDictKeyError):
        log.debug("Could not find product: %s", productslug)
        return bad_or_missing(request, _('The product you have requested does not exist.'))
        
    wish = ProductWish.objects.create_if_new(product, contact, details)
    url = urlresolvers.reverse('satchmo_wishlist_view')
    return HttpResponseRedirect(url)

def wishlist_add_ajax(request, template="shop/json.html"):
    data = {'errors': []}
    product = None
    formdata = request.POST.copy()
    productslug = formdata['productname']

    log.debug('WISHLIST AJAX: slug=%s', productslug)
    try:
        product, details = product_from_post(productslug, formdata)

    except (Product.DoesNotExist, MultiValueDictKeyError):
        log.warn("Could not find product: %s", productslug)
        product = None

    if not product:
        data['errors'].append(('product', _('The product you have requested does not exist.')))

        try:
            contact = Contact.objects.from_request(request)
        except Contact.DoesNotExist:
            log.warn("Could not find contact")
        
        if not contact:
            data['errors'].append(('contact', _('The contact associated with this request does not exist.')))
    else:
        data['id'] = product.id
        data['name'] = product.translated_name()
        
    if not data['errors']:
        wish = ProductWish.objects.create_if_new(product, contact, details)    
        data['results'] = _('Success')
    else:
       data['results'] = _('Error')

    encoded = JSONEncoder().encode(data)
    encoded = mark_safe(encoded)
    log.debug('WISHLIST AJAX: %s', data)
    
    return render_to_response(template, {'json' : encoded})

def wishlist_move_to_cart(request):
    wish, msg = _wish_from_post(request)
    if wish:
        cart = Cart.objects.from_request(request, create=True)
        try:
            cart.add_item(wish.product, number_added=1, details=wish.details)
        except CartAddProhibited, cap:
            msg = _("Wishlist product '%(product)s' could't be added to the cart. %(details)s") % {
                'product' : wish.product.translated_name, 
                'detail' : cap.message
                }
            return wishlist_view(request, message=msg)
            
        url = urlresolvers.reverse('satchmo_cart')
        satchmo_cart_changed.send(cart, cart=cart, request=request)
        return HttpResponseRedirect(url)
    else:
        return wishlist_view(request)    
    
def wishlist_remove(request):
    contact = Contact.objects.from_request(request)
    if not contact:
        return _wishlist_requires_login(request)
            
    success, msg = _wishlist_remove(request)
        
    return wishlist_view(request, message=msg)

def wishlist_remove_ajax(request, template="shop/json.html"):
    success, msg = _wishlist_remove(request)

    data = {
        'success' : success,
        'wishlist_message' : msg
    }
    encoded = JSONEncoder().encode(data)
    encoded = mark_safe(encoded)
    
    return render_to_response(template, {'json' : encoded})
    
def _wish_from_post(request):
    wid = request.POST.get('id', None)
    msg = ""
    wish = None
    if wid:
        try:
            wid = int(wid)
        except (TypeError, ValueError):
            msg = _("No such wishlist item.")
            wid = -1
        
        if wid > -1:
            contact = Contact.objects.from_request(request)
            if not contact:
                msg = _("You must be logged in to do that.")
            else:
                try:
                    wish = ProductWish.objects.get(contact=contact, id=wid)
                except ProductWish.DoesNotExist:
                    msg = _("No such wishlist item.")
        else:
            msg = _("No such wishlist item.")
            
    return wish, msg
    
def _wishlist_remove(request):
    success = False
    msg = ""
    wish, msg = _wish_from_post(request)
    if wish:
        success = True
        wish.delete()

    return success, msg

def _remove_wishes_on_order(order=None, **kwargs):
    log.debug("Caught order success, inspecting for wishes to remove.")
    if order:
        products = [item.product for item in order.orderitem_set.all()]
        for wish in ProductWish.objects.filter(contact = order.contact, product__in = products):
            log.debug('removing fulfilled wish for: %s', wish)
            wish.delete()

order_success.connect(_remove_wishes_on_order, sender=Order)

def _wishlist_requires_login(request):
    log.debug("wishlist requires login")
    ctx = RequestContext(request, {
        'login_url' : settings.LOGIN_URL
        })
    return render_to_response('wishlist/login_required.html',
                              context_instance=ctx)



from notmm.dbapi.orm import XdserverProxy
from test_support import unittest
from datetime import datetime

class QuerySetTestCase(unittest.TestCase):
    # Test case to emulate Django QuerySets using Schevo databases under
    # the hood.

    db = None
    model = None

class EntryManagerTestCase(QuerySetTestCase):

    db = XdserverProxy('blogs')
    model = db.BlogEntry 

    def test_intersection(self):
        # test getting all entries
        Q = self.db.Q
        q1 = Q.Match(self.model, 'pub_date', '<=', datetime.today())
        q2 = Q.Match(self.model, 'reviewed', '==', True)
        results = Q.Intersection(q1, q2)
        self.assertEqual(isinstance(results, Q.Intersection), True)




#
#   SERMEPA / ServiRed payments module for Satchmo
#
#   Author: Michal Salaban <michal (at) salaban.info>
#   with a great help of Fluendo S.A., Barcelona
#
#   Based on "Guia de comercios TPV Virtual SIS" ver. 5.18, 15/11/2008, SERMEPA
#   For more information about integration look at http://www.sermepa.es/
#
#   TODO: SERMEPA interface provides possibility of recurring payments, which
#   could be probably used for SubscriptionProducts. This module doesn't support it.
#
from livesettings import *
from django.utils.translation import ugettext_lazy as _

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_SERMEPA', 
    _('SERMEPA (ServiRed) Payment Module Settings'))

config_register_list(
    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.sermepa'
        ),
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'SERMEPA'
        ),
    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'Credit Card (via SERMEPA)',
        help_text = _('This will be passed to the translation utility'),
        ordering=10
        ),
    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = '^sermepa/',
        ordering=20
        ),
    BooleanValue(
        PAYMENT_GROUP,
        'LIVE',
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False,
        ordering=30
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_CURRENCY',
        description=_('Currency'),
        default='978',
        choices=[
            ('978', _("EUR (Euro)")),
            ('840', _("USD (US Dollar)")),
            ('826', _("GBP (British Pound)")),
            ('392', _("JPY (Japanese Yen)")),
            ],
        ordering=40
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_FUC',
        description=_('Merchant FUC'),
        help_text=_('Your FUC code'),
        ordering=50
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_TITULAR',
        description=_('Merchant title'),
        help_text=_('Description of your shop which will be visible on payment confirmation screen'),
        ordering=60,
        ),

    # signature
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_SIGNATURE_CODE',
        description=_('Signature code'),
        help_text=_('Your secret code used to sign transaction data'),
        ordering=100,
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_TEST_SIGNATURE_CODE',
        description=_('Test signature code'),
        help_text=_('Your secret code used to sign transaction data in test payments'),
        ordering=200,
        ),
    # terminal
    IntegerValue(
        PAYMENT_GROUP,
        'MERCHANT_TERMINAL',
        description=_('Terminal number'),
        default=1,
        ordering=110
        ),
    IntegerValue(
        PAYMENT_GROUP,
        'MERCHANT_TEST_TERMINAL',
        description=_('Test terminal number'),
        default=1,
        help_text=_('Terminal number used for test payments'),
        ordering=210
        ),
    # post url
    StringValue(
        PAYMENT_GROUP,
        'POST_URL',
        description=_('Post URL'),
        help_text=_('The SERMEPA URL for real transaction posting.'),
        default="https://sis.sermepa.es/sis/realizarPago",
        ordering=120
        ),
    StringValue(
        PAYMENT_GROUP,
        'POST_TEST_URL',
        description=_('Test Post URL'),
        help_text=_('The SERMEPA URL for test transaction posting.'),
        default="https://sis-t.sermepa.es:25443/sis/realizarPago",
        ordering=220
        ),

    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_URL_CALLBACK',
        description=_('Callback URL'),
        help_text=_('Callback URL for on-line notifications about payment progress'),
        default='SERMEPA_satchmo_checkout-notify_callback',
        ordering=300
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_URL_OK',
        description=_('OK URL'),
        help_text=_('URL for customer to return after successful payment'),
        default='SERMEPA_satchmo_checkout-success',
        ordering=310
        ),
    StringValue(
        PAYMENT_GROUP,
        'MERCHANT_URL_KO',
        description=_('Failure URL'),
        help_text=_('URL for customer to return after payment failure'),
        default='SERMEPA_satchmo_checkout-failure',
        ordering=320
        ),
    BooleanValue(
        PAYMENT_GROUP,
        'SSL',
        description=_("Use SSL for the module checkout pages?"),
        default=False,
        ordering=330
        ),
        
    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False)
    )



from livesettings import config_value
from satchmo_utils import load_module
import decimal

TWOPLACES = decimal.Decimal('0.01')

def get_tax_processor(order=None, user=None):
    modulename = config_value('TAX', 'MODULE')
    mod = load_module(u'%s.processor' % modulename)
    return mod.Processor(order=order, user=user)

def round_cents(x):
    return x.quantize(TWOPLACES, decimal.ROUND_FLOOR)



# Copyright (C) 2006 by Nokia Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import configparser
import glob
import os
import sys

from kiwi.environ import environ
from kiwi.desktopparser import DesktopParser
from kiwi.python import namedAny

from gazpacho.config import get_app_data_dir
from gazpacho.i18n import _

class Plugin(object):
    """Base class for Gazpacho plugins"""

    def __init__(self, name):
        self.name = name

    def activate(self, app):
        """
        This method is called to initialize the plugin.

        It can be called at gazpacho startup time or when the user
        manually enables the plugin in the configuration dialog
        """

    def deactivate(self):
        """
        This method is called to shutdown a plugin

        It's called when the user disables a plugin in the configuration
        dialog
        """

    def get_configuration_dialog(self, parent_window=None):
        """
        If the plugin has configuration options it should return
        a dialog in this method
        """

class PluginInfoError(Exception):
    pass

class PluginInfo(object):
    """
    This class parses and stores the metada of a .plugin file for a Plugin.

    The format of such files is:

    [Gazpacho Plugin]
    name = plugin_name
    title = short human readable string
    class = plugin.dotted.class.name
    description = text description
    author = author name and e-mail
    version = plugin version
    """
    def __init__(self, filename):
        self._filename = filename
        self._parser = DesktopParser()
        self._parser.read(filename)

        if not self._parser.has_section('Gazpacho Plugin'):
            msg = "The plugin file %s should have a [Gazpacho Plugin] section"
            raise PluginInfoError(msg % filename)

        self.name = self._read_value('name')
        self.title = self._read_value('title', self.name)
        self.class_name = self._read_value('class')
        self.description = self._read_value('description',
                                            _('No description available'))
        self.author = self._read_value('author',
                                       _('No author available'))
        self.version = self._read_value('version',
                                        _('No version available'))

    def _read_value(self, key, default=None):
        ret = None
        try:
            ret = self._parser.get('Gazpacho Plugin', key)
        except configparser.NoOptionError:
            if default:
                ret = default
            else:
                msg = ("The plugin file %s should have a %s option" %
                       (self._filename, key))
                raise PluginInfoError(msg)
        return ret

class PluginManager(object):
    """
    See interfaces.IPluginManager
    """

    def __init__(self):
        # in this dictionary we save instances of PluginInfo objects
        # it is indexed by their names
        self._plugins = {}

        # here we save Plugin objects for those plugins that are activated
        self._activated_plugins = []

    def load_plugins(self):
        # first load user plugins
        user_dir = os.path.join(get_app_data_dir('gazpacho'), 'plugins')
        self.load_plugins_dir(user_dir)

        for system_dir in environ.get_resource_paths('plugins'):
            self.load_plugins_dir(system_dir)

    def load_plugins_dir(self, dirname):
        if not os.path.exists(dirname):
            return

        # search for a .plugin file in every subdir of dirname
        for plugin_dir in os.listdir(dirname):
            if not os.path.isdir(os.path.join(dirname, plugin_dir)):
                continue

            files = glob.glob(os.path.join(dirname,
                                              plugin_dir,
                                              '*.plugin'))
            if len(files) > 1:
                print(('warning: plugin dir %s has more than one .plugin '
                       'file. It will not be loaded' % plugin_dir))
            elif len(files) == 1:
                metadata_file = files[0]

                pi = PluginInfo(metadata_file)

                if pi.name in self._plugins:
                    print(('The plugin %s was already loaded. Skipping' %
                           metadata_file))

                self._plugins[pi.name] = pi

        # add it to the python path so we can load plugins later
        if dirname not in sys.path:
            sys.path.append(dirname)

    def get_plugins(self):
        return list(self._plugins.values())

    def is_activated(self, plugin_name):
        for plugin in self._activated_plugins:
            if plugin.name == plugin_name:
                return True
        return False

    def activate_plugins(self, plugin_names, app):
        for pn in plugin_names:
            self.activate_plugin(pn, app)

    def activate_plugin(self, plugin_name, app):
        if plugin_name not in list(self._plugins.keys()):
            raise PluginInfoError("There is no plugin with the name %s" %
                                  plugin_name)

        klass = None
        plugin_info = self._plugins[plugin_name]
        try:
            klass = namedAny(plugin_info.class_name)
        except AttributeError:
            raise PluginInfoError('Error while activating plugin %s. '
                                  'Check the "class" option in the %s.plugin '
                                  'file' % (plugin_name, plugin_name))
        except ImportError:
            raise PluginInfoError('Error while importing plugin %s. '
                                  'Check the plugin file is a valid python '
                                  'module/package' % plugin_name)

        if klass:
            plugin = klass(plugin_info.name)
            plugin.activate(app)
            self._activated_plugins.append(plugin)

    def deactivate_all(self):
        for plugin in list(self._activated_plugins):
            self._deactivate_plugin_internal(plugin)

    def _deactivate_plugin_internal(self, plugin):
        """
        Deactivate the especified plugin.

        'plugin' should be a Plugin object and that's why this
        method is internal. There is no public way to get an
        instance of a Plugin class
        """
        plugin.deactivate()
        self._activated_plugins.remove(plugin)

    def deactivate_plugin(self, plugin_name):
        """
        Deactivate the plugin with that name
        """
        for p in self._activated_plugins:
            if p.name == plugin_name:
                self._deactivate_plugin_internal(p)
                return

    def show_plugin_preferences(self, plugin_name, parent_window):
        if plugin_name not in list(self._plugins.keys()):
            raise PluginInfoError("There is no plugin with the name %s" %
                                  plugin_name)

        for p in self._activated_plugins:
            if p.name == plugin_name:
                dialog = p.get_configuration_dialog(parent_window)
                if dialog:
                    dialog.run()
                    dialog.destroy()
                return

        raise PluginInfoError("There is no activated plugin with the name %s" %
                              plugin_name)



from django.utils.translation import ugettext as _
from livesettings import config_value
from product.models import Discount

from satchmo_store.mail import NoRecipientsException, send_store_mail, send_store_mail_template_decorator
from satchmo_store.shop.signals import order_confirmation_sender, order_notice_sender, ship_notice_sender

import logging
log = logging.getLogger('contact.notifications')

def order_success_listener(order=None, **kwargs):
    """Listen for order_success signal, and send confirmations"""
    if order:
        send_order_confirmation(order)
        send_order_notice(order)

def notify_on_ship_listener(sender, oldstatus="", newstatus="", order=None, **kwargs):
    """Listen for a transition to 'shipped', and notify customer."""

    if oldstatus != 'Shipped' and newstatus == 'Shipped':
        if order.is_shippable:
            send_ship_notice(order)

@send_store_mail_template_decorator('shop/email/order_complete')
def send_order_confirmation(order, template='', template_html=''):
    """Send an order confirmation mail to the customer.
    """

    try:
        sale = Discount.objects.by_code(order.discount_code, raises=True)
    except Discount.DoesNotExist:
        sale = None

    c = {'order': order, 'sale' : sale}
    subject = _("Thank you for your order from %(shop_name)s")

    send_store_mail(subject, c, template, [order.contact.email],
                    template_html=template_html, format_subject=True,
                    sender=order_confirmation_sender)

@send_store_mail_template_decorator('shop/email/order_placed_notice')
def send_order_notice(order, template='', template_html=''):
    """Send an order confirmation mail to the owner.
    """

    if config_value("PAYMENT", "ORDER_EMAIL_OWNER"):
        try:
            sale = Discount.objects.by_code(order.discount_code, raises=True)
        except Discount.DoesNotExist:
            sale = None

        c = {'order': order, 'sale' : sale}
        subject = _("New order on %(shop_name)s")

        eddresses = []
        more = config_value("PAYMENT", "ORDER_EMAIL_EXTRA")
        if more:
            eddresses = set([m.strip() for m in more.split(',')])
            eddresses = [e for e in eddresses if e]

        try:
            send_store_mail(subject, c, template, eddresses,
                            template_html=template_html, format_subject=True,
                            send_to_store=True, sender=order_notice_sender)
        except NoRecipientsException:
            log.warn("No shop owner email specified, skipping owner_email")
            return

# TODO add html email template
def send_ship_notice(order, template='shop/email/order_shipped.txt', template_html=''):
    """Send an order shipped mail to the customer.
    """

    log.debug('ship notice on %s', order)

    c = {'order': order}
    subject = _("Your order from %(shop_name)s has shipped")

    send_store_mail(subject, c, template, [order.contact.email],
                    format_subject=True, template_html=template_html,
                    sender=ship_notice_sender)



from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from satchmo_utils.thumbnail.utils import make_admin_thumbnail

class AdminImageWithThumbnailWidget(forms.FileInput):
    """
    A FileField Widget that shows its current image as a thumbnail if it has one.
    """
    def __init__(self, attrs={}):
        super(AdminImageWithThumbnailWidget, self).__init__(attrs)
        
    def render(self, name, value, attrs=None):
        output = []
        if value and hasattr(value, "url"):
            thumb = make_admin_thumbnail(value.url)
            if not thumb:
                thumb = value.url
            output.append('<img src="%s" /><br/>%s<br/> %s ' % \
                (thumb, value.url, _('Change:')))
        output.append(super(AdminImageWithThumbnailWidget, self).render(name, value, attrs))
        return mark_safe(u''.join(output))



# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os

import gobject
import gtk
from kiwi.environ import environ

from gazpacho import util
from gazpacho.cursor import Cursor
from gazpacho.interfaces import BaseWidgetAdaptor
from gazpacho.gadget import Gadget, load_gadget_from_widget

class CreationCancelled(Exception):
    """This is raised when the user cancels the creation of a widget"""
    pass

def create_pixbuf(generic_name, palette_name, library_name, resource_path):
    pixbuf = None

    if generic_name and palette_name:
        filename = '%s.png' % generic_name
        if resource_path is not None:
            path = environ.find_resource('resource',
                                         os.path.join(library_name, filename))
        else:
            path = environ.find_resource('pixmap', filename)

        if path is not None:
            try:
                pixbuf = gtk.gdk.pixbuf_new_from_file(path)
            except gobject.GError:
                pass

    # If we cannot find the icon and it's going to be shown in the palette
    # Print a warning and create an icon of a missing image instead
    if not pixbuf and palette_name:
        print('Warning: Icon for %s is missing' % palette_name)
        da = gtk.DrawingArea()
        pixbuf = da.render_icon(gtk.STOCK_MISSING_IMAGE,
                                gtk.ICON_SIZE_MENU)
    return pixbuf

def create_icon(pixbuf):
    image = gtk.Image()
    image.set_from_pixbuf(pixbuf)
    return image

class WidgetAdaptor(object, BaseWidgetAdaptor):
    """Base class for all widget adaptors

    A widget adaptor is the main class of a library plugin. There is a widget
    adaptor for every widget in the library that needs to do anything special
    upon certain events in Gazpacho. See this class virtual methods for
    examples of such special events.

    The rest of widgets (e.g. regular widgets) use just an instance of this
    base class, which provides the basic functionality.

    Check gazpacho/widgets/base/base.py for examples of widget adaptors.

    @ivar type: GTK type for this widget. Used to create widgets with
                gobject.new()
    @ivar type_name: type_name is used when saving the xml
    @ivar editor_name: Name displayed in the editor
    @ivar name: Optional identifier of adaptor, GType name of type will
                be used if not specified
    @ivar generic_name: generic_name is used to create default widget names
    @ivar palette_name: palette_name is used in the palette
    @ivar tooltip: the tooltip is shown in the palette
    @ivar library:
    @ivar icon:
    @ivar cursor:
    @ivar pixbuf
    @ivar default: default widget as created in gobject.new(). It is used to
                   decide if any property has changed
    @ivar default_child: for packing properties we need to save the
                         default child too
    """

    type = None
    name = None

    def __init__(self, type_name, generic_name, palette_name, library,
                 resource_path, tooltip):
        """
        @param type_name:
        @param generic_name:
        @param palette_name:
        @param library:
        @param resource:
        @param tooltip:
        """
        self._default = None
        self._default_child = {}
        #self.type = None
        #self.name = None
        if self.type:
            if not issubclass(self.type, gtk.Widget):
                raise TypeError("%s.type needs to be a gtk.Widget subclass")
            type_name = gobject.type_name(self.type)
        else:
            try:
                self.type = gobject.type_from_name(type_name)
            except RuntimeError:
                raise AttributeError(
                    "There is no registered widget called %s" % type_name)
        self.type_name = type_name

        self.editor_name = type_name
        self.generic_name = generic_name
        self.palette_name = palette_name
        self.library = library
        self.tooltip = tooltip
        self.pixbuf = create_pixbuf(generic_name, palette_name, library.name,
                                    resource_path)
        self.icon = create_icon(self.pixbuf)
        self.cursor = Cursor.create(self.pixbuf)

    # Virtual methods

    def create(self, context, interactive=True):
        """
        Called when creating a widget, it should return an instance
        of self.type
        """
        return self.library.create_widget(self.type)

    def post_create(self, context, widget, interactive=True):
        """
        Called after all initialization is done in the creation process.

        It takes care of creating the gadgets associated with internal
        children. It's also the place to set sane defaults, e.g. set the size
        of a window.
        """

    def fill_empty(self, context, widget):
        """
        After the widget is created this function is called to put one or
        more placeholders in it. Only useful for container widgets
        """

    def replace_child(self, context, old_widget, new_widget, parent_widget):
        """
        Called when the user clicks on a placeholder having a palette icon
        selected. It replaced a placeholder for the new widget.

        It's also called in the reverse direction (replacing a widget for a
        placeholder) when removing a widget or undoing a create operation.
        """

    def save(self, context, widget):
        """
        Prepares the widget to be saved. Basically this mean setting all
        the gazpacho.widgets for internal children so the filewriter can
        iterate through them and write them correctly.
        """

    def load(self, context, widget):
        """
        Build a gadget from a widget

        The loading is a two step process: first we get the widget tree from
        gazpacho.loader or libglade and then we create the gadgets
        from that widget tree. This function is responsable of
        the second step of this loading process
        """
        from gazpacho.placeholder import Placeholder
        project = context.get_project()
        gadget = Gadget.load(widget, project)

        # create the children
        if isinstance(widget, gtk.Container):
            for child in util.get_all_children(widget):
                if isinstance(child, Placeholder):
                    continue
                load_gadget_from_widget(child, project)

        # if the widget is a toplevel we need to attach the accel groups
        # of the application
        if gadget.is_toplevel():
            gadget.setup_toplevel()

        return gadget

    def button_release(self, context, widget, event):
        """
        Called when a button release event occurs in the widget.

        Note that if the widget is a windowless widget the event is actually
        produced in its first parent with a gdk window so you will probably
        want to translate the event coordinates.
        """
        return False # keep events propagating the usual way

    def motion_notify(self, context, widget, event):
        """
        Called when the mouse is moved on the widget.

        Note that if the widget is a windowless widget the event is actually
        produced in its first parent with a gdk window so you will probably
        want to translate the event coordinates.
        """
        return False # keep events propagating the usual way

    def get_children(self, context, widget):
        """
        To get the list of children for a widget, overridable by subclasses
        """
        return []

    def delete(self, context, widget):
        """
        Called when a widget has been deleted.
        """
        pass

    def restore(self, context, widget, data):
        """
        Called when a widget that were previously deleted has been
        restored again.
        """
        pass

    # Public API

    def get_default(self):
        return self._default

    def get_default_prop_value(self, prop, parent_type):
        """
        This is responsible for accessing the default value of a property.
        And what we consider default is the value assigned to an
        object immediatelly after it's called gobject.new.
        parent_name is used for packing properties
        @param prop: a property
        @param parent_type: parent gtype
        """
        if not prop.child:
            if not self._default:
                self._default = self.library.create_widget(self.type)

            default_value = prop.get_default(self._default)
        else:
            # Child properties are trickier, since we do not only
            # Need to create the object, but the parent too
            if not parent_type in self._default_child:
                parent = gobject.new(parent_type)
                child = gobject.new(self.type_name)
                self._default_child[parent_type] = parent, child
                parent.add(child)
            else:
                parent, child = self._default_child[parent_type]

            default_value = parent.child_get(child, prop.name)[0]

        return default_value

    def list_signals(self):
        """
        @returns: a list of signals
        """
        signals = []

        gobject_gtype = gobject.GObject.__gtype__
        gtype = self.type
        while True:
            kn = gobject.type_name(gtype)
            signal_names = list(gobject.signal_list_names(gtype))
            signal_names.sort()
            signals += [(name, kn) for name in signal_names]
            if gtype == gobject_gtype:
                break

            gtype = gobject.type_parent(gtype)

        return signals

    def is_toplevel(self):
        """
        @returns: True if it's a toplevel, False otherwise
        """
        return gobject.type_is_a(self.type, gtk.Window)



# Copyright (C) 2005 by Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import sys
from configparser import ConfigParser

MAX_RECENT = 10

"""[General]
lastdirectory=..
bugname=
bugmail=

[RecentProjects]
project0=..
project1=..

# in the plugins section each key represents a plugin
# and if the value is True that means the plugin should
# be activated when Gazpacho starts up
[Plugins]
foo=True
bar=True
"""

class ConfigError(Exception):
    pass

class BaseConfig:
    def __init__(self, filename):
        self._filename = filename
        self._config = ConfigParser()
        self.open(self._filename)

    def open(self, filename):
        if os.path.exists(filename):
            self._config.read(filename)
        self._filename = filename

    def has_option(self, name, section='General'):
        return self._config.has_option(section, name)

    def get_option(self, name, section='General'):
        if not section in self.sections:
            raise ConfigError('Invalid section: %s' % section)

        if self._config.has_option(section, name):
            return self._config.get(section, name)

        raise ConfigError('%s does not have option: %s' %
                          (self._filename, name))

    def set_option(self, name, value, section='General'):
        if not section in self.sections:
            raise ConfigError('Invalid section: %s' % section)

        if not self._config.has_section(section):
            self._config.add_section(section)

        self._config.set(section, name, value)

    def save(self):
        filename = self._filename
        fp = open(filename, 'w')
        self._config.write(fp)

def get_app_data_dir(appname):
    if sys.platform == 'win32':
        try:
            # C:\Documents and Settings\<user>\Application Data
            from win32com.shell.shell import SHGetFolderPath
            try:
                from win32com.shellcon import CSIDL_APPDATA
                folder_type = CSIDL_APPDATA
            except ImportError:
                folder_type = 0x001a
            app_data_dir = SHGetFolderPath(0, folder_type, 0, 0)
        except ImportError:
            # default to C:\
            app_data_dir = 'C:\\'
        return os.path.join(app_data_dir, appname)
    else:
        return os.path.join(os.path.expanduser('~'), '.' + appname)

class GazpachoConfig(BaseConfig):
    sections = ['General', 'RecentProjects', 'Plugins']
    def __init__(self):
        self.recent_projects = []
        self.lastdirectory = None
        self.plugins = []
        BaseConfig.__init__(self, self.get_filename())

    def get_filename(self):
        projectdir = get_app_data_dir('gazpacho')
        if not os.path.exists(projectdir):
            os.mkdir(projectdir)
        return os.path.join(projectdir, 'config')

    def open(self, filename):
        BaseConfig.open(self, filename)

        if self.has_option('lastdirectory'):
            self.lastdirectory = self.get_option('lastdirectory')

        i = 0
        while True:
            name = 'project%d' % i
            if not self.has_option(name, 'RecentProjects'):
                break

            self.recent_projects.append(self.get_option(name, 'RecentProjects'))
            i += 1

    def save(self):
        self.set_option('lastdirectory', self.lastdirectory)
        for i, project in enumerate(self.recent_projects):
            self.set_option('project%d' % i, project, 'RecentProjects')

        BaseConfig.save(self)

    def set_lastdirectory(self, filename):
        if not os.path.isdir(filename):
            filename = os.path.dirname(filename)
        self.lastdirectory = filename

    def add_recent_project(self, path):
        """
        Add a project to the recent project list.

        @param path: the project path
        @type path: str
        """
        if not path:
            return

        if path in self.recent_projects:
            self.recent_projects.remove(path)
            self.recent_projects.insert(0, path)
            return

        self.recent_projects.insert(0, path)
        if len(self.recent_projects) > MAX_RECENT:
            self.recent_projects = self.recent_projects[:MAX_RECENT]

    def set_bugname(self, name):
        self.set_option('bugname', name)

    def get_bugname(self):
        if self.has_option('bugname'):
            return self.get_option('bugname')
        return ''

    def set_bugmail(self, name):
        self.set_option('bugmail', name)

    def get_bugmail(self):
        if self.has_option('bugmail'):
            return self.get_option('bugmail')
        return ''

    def set_plugin(self, plugin, activated):
        if not self._config.has_section('Plugins'):
            self._config.add_section('Plugins')

        self.set_option(plugin, activated, 'Plugins')

    def get_plugins(self):
        plugins = []
        if self._config.has_section('Plugins'):
            section = self._config.items('Plugins')
            plugins = [name for name, value in section
                                if self._config.getboolean('Plugins', name)]
        return plugins

config = GazpachoConfig()



# -*- coding: utf-8 -*-
"""utilities using the threaded multihost system.

:Authors:
    - Herbert Poul http://sct.sphene.net
    - Bruce Kroeze http://gosatchmo.com
"""
"""
New BSD License
===============
Copyright (c) 2008, Bruce Kroeze http://solidsitesolutions.com

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of SolidSiteSolutions LLC, Zefamily LLC nor the names of its 
      contributors may be used to endorse or promote products derived from this 
      software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
__docformat__="restructuredtext"

from django.conf import settings
import threadlocals
import logging

log = logging.getLogger('threaded_multihost.utils')

def is_secure():
    req = threadlocals.get_current_request()
    if not req:
        log.warning("No request found while doing media_tags.  Are you missing the ThreadLocalMiddleware?")
        return False
    return req.is_secure()

def current_media_url():
    if not is_secure():
        return settings.MEDIA_URL
    else:
        try:
            return settings.MEDIA_SECURE_URL
        except AttributeError:
            return settings.MEDIA_URL.replace('http://','https://')


from livesettings import *
from django.utils.translation import ugettext_lazy as _


PRODUCTS = config_get('PRODUCT', 'PRODUCT_TYPES')
PRODUCTS.add_choice(('giftcertificate::GiftCertificateProduct', _('Gift Certificate')))

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_GIFTCERTIFICATE', 
    _('Gift Certificate Settings'))

config_register_list(
    BooleanValue(PAYMENT_GROUP, 
        'SSL', 
        description=_("Use SSL for the checkout pages?"), 
        default=False),
        
    StringValue(PAYMENT_GROUP,
        'CHARSET',
        description=_("Character Set"),
        default="BCDFGHKPRSTVWXYZbcdfghkprstvwxyz23456789",
        help_text=_("The characters allowable in randomly-generated certficate codes.  No vowels means no unfortunate words.")),
        
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'GIFTCERTIFICATE'),
        
    StringValue(PAYMENT_GROUP,
        'FORMAT',
        description=_('Code format'),
        default="^^^^-^^^^-^^^^",
        help_text=_("Enter the format for your cert code.  Use a '^' for the location of a randomly generated character.")),
        
    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.giftcertificate'),

    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'Gift Certificate',
        help_text = _('This will be passed to the translation utility')),
        
    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False),

    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = r'^giftcertificate/'),
        
    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False)
)



'''
This module provides support for defining properties on your entities. It both
provides, the `Property` class which acts as a building block for common
properties such as fields and relationships (for those, please consult the
corresponding modules), but also provides some more specialized properties,
such as `ColumnProperty` and `Synonym`. It also provides the GenericProperty
class which allows you to wrap any SQLAlchemy property, and its DSL-syntax
equivalent: has_property_.

`has_property`
--------------
The ``has_property`` statement allows you to define properties which rely on
their entity's table (and columns) being defined before they can be declared
themselves. The `has_property` statement takes two arguments: first the name of
the property to be defined and second a function (often given as an anonymous
lambda) taking one argument and returning the desired SQLAlchemy property. That
function will be called whenever the entity table is completely defined, and
will be given the .c attribute of the entity as argument (as a way to access
the entity columns).

Here is a quick example of how to use ``has_property``.

.. sourcecode:: python

    class OrderLine(Entity):
        has_field('quantity', Float)
        has_field('unit_price', Float)
        has_property('price',
                     lambda c: column_property(
                         (c.quantity * c.unit_price).label('price')))
'''

from elixir.statements import PropertyStatement
from sqlalchemy.orm import column_property, synonym

__doc_all__ = ['EntityBuilder', 'Property', 'GenericProperty',
               'ColumnProperty']

class EntityBuilder(object):
    '''
    Abstract base class for all entity builders. An Entity builder is a class
    of objects which can be added to an Entity (usually by using special
    properties or statements) to "build" that entity. Building an entity,
    meaning to add columns to its "main" table, create other tables, add
    properties to its mapper, ... To do so an EntityBuilder must override the
    corresponding method(s). This is to ensure the different operations happen
    in the correct order (for example, that the table is fully created before
    the mapper that use it is defined).
    '''
    def create_pk_cols(self):
        pass

    def create_non_pk_cols(self):
        pass

    def before_table(self):
        pass

    def create_tables(self):
        '''
        Subclasses may override this method to create tables.
        '''

    def after_table(self):
        pass

    def create_properties(self):
        '''
        Subclasses may override this method to add properties to the involved
        entity.
        '''

    def before_mapper(self):
        pass

    def after_mapper(self):
        pass

    def finalize(self):
        pass

    # helper methods
    def add_table_column(self, column):
        self.entity._descriptor.add_column(column)

    def add_mapper_property(self, name, prop):
        self.entity._descriptor.add_property(name, prop)

    def add_mapper_extension(self, ext):
        self.entity._descriptor.add_mapper_extension(ext)


class CounterMeta(type):
    '''
    A simple meta class which adds a ``_counter`` attribute to the instances of
    the classes it is used on. This counter is simply incremented for each new
    instance.
    '''
    counter = 0

    def __call__(self, *args, **kwargs):
        instance = type.__call__(self, *args, **kwargs)
        instance._counter = CounterMeta.counter
        CounterMeta.counter += 1
        return instance


class Property(EntityBuilder):
    '''
    Abstract base class for all properties of an Entity.
    '''
    __metaclass__ = CounterMeta

    def __init__(self, *args, **kwargs):
        self.entity = None
        self.name = None

    def attach(self, entity, name):
        """Attach this property to its entity, using 'name' as name.

        Properties will be attached in the order they were declared.
        """
        self.entity = entity
        self.name = name

        # register this property as a builder
        entity._descriptor.builders.append(self)

    def __repr__(self):
        return "Property(%s, %s)" % (self.name, self.entity)


class GenericProperty(Property):
    '''
    Generic catch-all class to wrap an SQLAlchemy property.

    .. sourcecode:: python

        class OrderLine(Entity):
            quantity = Field(Float)
            unit_price = Field(Numeric)
            price = GenericProperty(lambda c: column_property(
                             (c.quantity * c.unit_price).label('price')))
    '''

    def __init__(self, prop, *args, **kwargs):
        super(GenericProperty, self).__init__(*args, **kwargs)
        self.prop = prop
        #XXX: move this to Property?
        self.args = args
        self.kwargs = kwargs

    def create_properties(self):
        if hasattr(self.prop, '__call__'):
            prop_value = self.prop(self.entity.table.c)
        else:
            prop_value = self.prop
        prop_value = self.evaluate_property(prop_value)
        self.add_mapper_property(self.name, prop_value)

    def evaluate_property(self, prop):
        if self.args or self.kwargs:
            raise Exception('superfluous arguments passed to GenericProperty')
        return prop


class ColumnProperty(GenericProperty):
    '''
    A specialized form of the GenericProperty to generate SQLAlchemy
    ``column_property``'s.

    It takes a function (often given as an anonymous lambda) as its first
    argument. Other arguments and keyword arguments are forwarded to the
    column_property construct. That first-argument function must accept exactly
    one argument and must return the desired (scalar-returning) SQLAlchemy
    ClauseElement.

    The function will be called whenever the entity table is completely
    defined, and will be given
    the .c attribute of the table of the entity as argument (as a way to
    access the entity columns). The ColumnProperty will first wrap your
    ClauseElement in an
    "empty" label (ie it will be labelled automatically during queries),
    then wrap that in a column_property.

    .. sourcecode:: python

        class OrderLine(Entity):
            quantity = Field(Float)
            unit_price = Field(Numeric)
            price = ColumnProperty(lambda c: c.quantity * c.unit_price,
                                   deferred=True)

    Please look at the `corresponding SQLAlchemy
    documentation <http://www.sqlalchemy.org/docs/05/mappers.html
    #sql-expressions-as-mapped-attributes>`_ for details.
    '''

    def evaluate_property(self, prop):
        return column_property(prop.label(None), *self.args, **self.kwargs)


class Synonym(GenericProperty):
    '''
    This class represents a synonym property of another property (column, ...)
    of an entity.  As opposed to the `synonym` kwarg to the Field class (which
    share the same goal), this class can be used to define a synonym of a
    property defined in a parent class (of the current class). On the other
    hand, it cannot define a synonym for the purpose of using a standard python
    property in queries. See the Field class for details on that usage.

    .. sourcecode:: python

    class Person(Entity):
        name = Field(String(30))
        primary_email = Field(String(100))
        email_address = Synonym('primary_email')

    class User(Person):
        user_name = Synonym('name')
        password = Field(String(20))
    '''

    def evaluate_property(self, prop):
        return synonym(prop, *self.args, **self.kwargs)

#class Composite(GenericProperty):
#    def __init__(self, prop):
#        super(GenericProperty, self).__init__()
#        self.prop = prop

#    def evaluate_property(self, prop):
#        return composite(prop.label(self.name))

#start = Composite(Point, lambda c: (c.x1, c.y1))

#mapper(Vertex, vertices, properties={
#    'start':composite(Point, vertices.c.x1, vertices.c.y1),
#    'end':composite(Point, vertices.c.x2, vertices.c.y2)
#})


has_property = PropertyStatement(GenericProperty)




from django.contrib import admin
from django.forms import models, ValidationError
from django.utils.translation import get_language, ugettext_lazy as _
from models import TaxRate

class TaxRateForm(models.ModelForm):

    def clean(self):
        tax_zone = self.cleaned_data['taxZone']
        tax_country = self.cleaned_data['taxCountry']
        if tax_zone and not tax_country or not tax_zone and tax_country:
            return super(TaxRateForm, self).clean()
        else:
            raise ValidationError(_("You must choose a zone or a country."))
        
class TaxRateOptions(admin.ModelAdmin):
    list_display = ("taxClass", "taxZone", "taxCountry", "display_percentage")
    form = TaxRateForm
    
admin.site.register(TaxRate, TaxRateOptions)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .response import HTTPResponse

class HTTPException(HTTPResponse, BaseException):
    status_int = 500
    etag = None
HTTPServerError = HTTPException

class HTTPNotFound(HTTPException):
    """HTTP 404 (Not Found)"""
    status_int = 404
HTTPNotFoundResponse = HTTPNotFound    

class HTTPNotModified(HTTPNotFound):
    status_int = 304
class HTTPClientError(HTTPException):
    status_int = 400 # bad request
class HTTPUnauthorized(HTTPClientError):
    status_int = 401
class HTTPForbidden(HTTPClientError):
    status_int = 403
class HTTPFoundResponse(HTTPResponse):
    """HTTP 302 (Found)
    
    Requires one param: 
    * location: a URI string.
    """
    status_int = 302
    
    def __init__(self, location, **kwargs):
        #kwargs['status'] = str(self.status_int)
        # Displays a short message to the user with a link to
        # the given resource URI
        # TODO: Use a template for this.
        kwargs['content'] = '''\
        <html>
        <p>Click here to follow this redirect: <a href=\"%s\">Link</a></p>
        </html>'''%location
        kwargs['mimetype'] = 'text/html'
        super(HTTPFoundResponse, self).__init__(**kwargs)
        
        #self.location = location     # location to redirect to

        #self.initial_kwargs = kwargs 
        self.http_headers.append(('Location', location)) #.split('?next=')[1]
        self.http_headers.append(('Cache-Control', 'no-cache')) # Prevent caching redirects

    def __call__(self, env, start_response):
        start_response(self.status_code, self.headers)
        return self.app_iter

HTTPRedirectResponse = HTTPFoundResponse # alias

class HTTPSeeOtherResponse(HTTPFoundResponse):
    """HTTP 303 (See Other)"""
    status_int = 303

class HTTPNotModifiedResponse(HTTPResponse):
    """HTTP 304 (Not Modified)"""
    status_int = 304

    def __init__(self, *args, **kwargs):
        super(HTTPNotModifiedResponse, self).__init__(*args, **kwargs)
        
        self.http_headers['Date'] = datetime.now().ctime()
            

        del self.http_headers['Content-Type']
    
        self.content = '';


class HTTPUnauthorizedResponse(HTTPResponse):
    """HTTP 401 (Unauthorized)"""
    status_int = 401

class HTTPForbiddenResponse(HTTPResponse):
    """HTTP 403 (Forbidden)"""
    status_int = 403

HTTPForbidden = HTTPForbiddenResponse





from decimal import Decimal
from django.core import urlresolvers
from django.http import HttpResponseRedirect, HttpResponse, Http404
from django.shortcuts import render_to_response
from django.template import RequestContext, loader
from django.utils.datastructures import MultiValueDictKeyError
from django.utils.safestring import mark_safe
from django.utils import simplejson
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from livesettings import config_value
from product.models import Product, OptionManager
from product.utils import find_best_auto_discount
from product.views import optionids_from_post
from satchmo_store.shop import forms
from satchmo_store.shop.exceptions import CartAddProhibited
from satchmo_store.shop.models import Cart, CartItem, NullCart, NullCartItem
from satchmo_store.shop.signals import satchmo_cart_changed, satchmo_cart_add_complete, satchmo_cart_details_query
from satchmo_utils.numbers import RoundedDecimalError, round_decimal
from satchmo_utils.views import bad_or_missing
import logging

try:
    from django.utils.simplejson.encoder import JSONEncoder
except ImportError:
    from simplejson.encoder import JSONEncoder

log = logging.getLogger('shop.views.cart')


def _json_response(data, error=False, template="shop/json.html"):
    response = HttpResponse(loader.render_to_string(template, 
        {'json':mark_safe(simplejson.dumps(data))}), mimetype='application/javascript')
    
    if error:
        response.status_code = 400
    
    return response

def decimal_too_big(quantity):
    """
    Helper to make sure the decimal number isn't too big to process.
    This does not validate whether or not the decimal is valid just that a
    Decimal is too large.
    """
    try:
        if Decimal(quantity) > Decimal("100000000000"):
            return True
        else:
            return False
    except:
        return False


def _set_quantity(request, force_delete=False):
    """Set the quantity for a specific cartitem.
    Checks to make sure the item is actually in the user's cart.
    """
    cart = Cart.objects.from_request(request, create=False)
    if isinstance(cart, NullCart):
        return (False, None, None, _("No cart to update."))

    cartplaces = config_value('SHOP', 'CART_PRECISION')

    if decimal_too_big(request.POST.get('quantity', 0)):
        return (False,cart,None,_("Bad quantity."))

    if force_delete:
        qty = Decimal('0')
    else:
        try:
            roundfactor = config_value('SHOP', 'CART_ROUNDING')
            qty = round_decimal(request.POST.get('quantity', 0), places=cartplaces, roundfactor=roundfactor, normalize=True)
        except RoundedDecimalError, P:
            return (False, cart, None, _("Bad quantity."))

        if qty < Decimal('0'):
            qty = Decimal('0')

    try:
        itemid = int(request.POST.get('cartitem'))
    except (TypeError, ValueError):
        return (False, cart, None, _("Bad item number."))

    try:
        cartitem = CartItem.objects.get(pk=itemid, cart=cart)
    except CartItem.DoesNotExist:
        return (False, cart, None, _("No such item in your cart."))

    if qty == Decimal('0'):
        cartitem.delete()
        cartitem = NullCartItem(itemid)
    else:
        from satchmo_store.shop.models import Config
        config = Config.objects.get_current()
        if config_value('PRODUCT','NO_STOCK_CHECKOUT') == False:
            stock = cartitem.product.items_in_stock
            log.debug('checking stock quantity.  Have %d, need %d', stock, qty)
            if stock < qty:
                return (False, cart, cartitem, _("Not enough items of '%s' in stock.") % cartitem.product.translated_name())

        cartitem.quantity = round_decimal(qty, places=cartplaces)
        cartitem.save()

    satchmo_cart_changed.send(cart, cart=cart, request=request)
    return (True, cart, cartitem, "")

def display(request, cart=None, error_message='', default_view_tax=None):
    """Display the items in the cart."""

    if default_view_tax is None:
        default_view_tax = config_value('TAX', 'DEFAULT_VIEW_TAX')

    if not cart:
        cart = Cart.objects.from_request(request)

    if cart.numItems > 0:
        products = [item.product for item in cart.cartitem_set.all()]
        sale = find_best_auto_discount(products)
    else:
        sale = None

    context = RequestContext(request, {
        'cart': cart,
        'error_message': error_message,
        'default_view_tax' : default_view_tax,
        'sale' : sale,
        })
    return render_to_response('shop/cart.html', context_instance=context)

display = never_cache(display)

def add(request, id=0, redirect_to='satchmo_cart'):
    """Add an item to the cart."""
    log.debug('FORM: %s', request.POST)
    formdata = request.POST.copy()
    productslug = None

    cartplaces = config_value('SHOP', 'CART_PRECISION')
    roundfactor = config_value('SHOP', 'CART_ROUNDING')


    if formdata.has_key('productname'):
        productslug = formdata['productname']
    try:
        product, details = product_from_post(productslug, formdata)

        if not (product and product.active):
            log.debug("product %s is not active" % productslug)
            return bad_or_missing(request, _("That product is not available at the moment."))
        else:
            log.debug("product %s is active" % productslug)

    except (Product.DoesNotExist, MultiValueDictKeyError):
        log.debug("Could not find product: %s", productslug)
        return bad_or_missing(request, _('The product you have requested does not exist.'))

    # First we validate that the number isn't too big.
    if decimal_too_big(formdata['quantity']):
        return _product_error(request, product, _("Please enter a smaller number."))

    # Then we validate that we can round it appropriately.
    try:
        quantity = round_decimal(formdata['quantity'], places=cartplaces, roundfactor=roundfactor)
    except RoundedDecimalError, P:
        return _product_error(request, product,
            _("Invalid quantity."))

    if quantity <= Decimal('0'):
        return _product_error(request, product,
            _("Please enter a positive number."))

    cart = Cart.objects.from_request(request, create=True)
    # send a signal so that listeners can update product details before we add it to the cart.
    satchmo_cart_details_query.send(
            cart,
            product=product,
            quantity=quantity,
            details=details,
            request=request,
            form=formdata
            )
    try:
        added_item = cart.add_item(product, number_added=quantity, details=details)

    except CartAddProhibited, cap:
        return _product_error(request, product, cap.message)

    # got to here with no error, now send a signal so that listeners can also operate on this form.
    satchmo_cart_add_complete.send(cart, cart=cart, cartitem=added_item, product=product, request=request, form=formdata)
    satchmo_cart_changed.send(cart, cart=cart, request=request)

    if request.is_ajax():
        data = {}
        data['id'] = product.id
        data['name'] = product.translated_name()
        data['cart_count'] = str(round_decimal(cart.numItems, 2))
        data['cart_total'] = str(cart.total)
        # Legacy result, for now
        data['results'] = _("Success")
        log.debug('CART AJAX: %s', data)
        
        return _json_response(data)
    else:
        url = urlresolvers.reverse(redirect_to)
        return HttpResponseRedirect(url)

def add_ajax(request, id=0, template="shop/json.html"):
    # Allow for legacy apps to still use this url
    if not request.META.has_key('HTTP_X_REQUESTED_WITH'):
        request.META['HTTP_X_REQUESTED_WITH'] = 'XMLHttpRequest'
        
    return add(request, id)

def add_multiple(request, redirect_to='satchmo_cart', products=None, template="shop/multiple_product_form.html"):
    """Add multiple items to the cart.
    """
    if request.method == "POST":
        log.debug('FORM: %s', request.POST)
        formdata = request.POST.copy()
        form = forms.MultipleProductForm(formdata, products=products)

        if form.is_valid():
            cart = Cart.objects.from_request(request, create=True)
            form.save(cart, request)
            satchmo_cart_changed.send(cart, cart=cart, request=request)

            url = urlresolvers.reverse(redirect_to)
            return HttpResponseRedirect(url)
    else:
        form = forms.MultipleProductForm(products=products)

    return render_to_response(template, context_instance=RequestContext(request, {'form' : form}))

def agree_terms(request):
    """Agree to terms"""
    if request.method == "POST":
        if request.POST.get('agree_terms', False):
            url = urlresolvers.reverse('satchmo_checkout-step1')
            return HttpResponseRedirect(url)

    return display(request, error_message=_('You must accept the terms and conditions.'))

def remove(request):
    """Remove an item from the cart."""
    if not request.POST:
        # Should be a POST request
        return bad_or_missing(request, "Please use a POST request")

    success, cart, cartitem, errors = _set_quantity(request, force_delete=True)
    
    if request.is_ajax():
        if errors:
            return _json_response({'errors': errors, 'results': _("Error")}, True)
        else:
            return _json_response({
                'cart_total': str(cart.total),
                'cart_count': str(cart.numItems),
                'item_id': cartitem.id,
                'results': success, # Legacy
            })
    else:
        if errors:
            return display(request, cart=cart, error_message=errors)
        else:
            url = urlresolvers.reverse('satchmo_cart')
            return HttpResponseRedirect(url)

def remove_ajax(request, template="shop/json.html"):
    """Remove an item from the cart. Returning JSON formatted results."""
    # Allow for legacy apps to still use this url
    if not request.META.has_key('HTTP_X_REQUESTED_WITH'):
        request.META['HTTP_X_REQUESTED_WITH'] = 'XMLHttpRequest'
        
    return remove(request)

def set_quantity(request):
    """Set the quantity for a cart item.

    Intended to be called via the cart itself, returning to the cart after done.
    """
    cart_url = urlresolvers.reverse('satchmo_cart')

    if not request.POST:
        return HttpResponseRedirect(cart_url)

    success, cart, cartitem, errors = _set_quantity(request)
            
    if request.is_ajax():
        if errors:
            return _json_response({'errors': errors, 'results': _("Error")}, True)
        else:
            return _json_response({
                'item_id': cartitem.id,
                'item_qty': str(cartitem.quantity) or "0",
                'item_price': str(cartitem.line_total) or "0.00",
                'cart_total': str(cart.total) or "0.00",
                'cart_count': str(cart.numItems) or '0',
            })
        
    else:
        if success:
            return HttpResponseRedirect(cart_url)
        else:
            return display(request, cart = cart, error_message = errors)
        
        
def set_quantity_ajax(request, template="shop/json.html"):
    """Set the quantity for a cart item, returning results formatted for handling by script.
    Kept for legacy apps.
    """
    if not request.META.has_key('HTTP_X_REQUESTED_WITH'):
        request.META['HTTP_X_REQUESTED_WITH'] = 'XMLHttpRequest'
        
    return set_quantity(request)

def product_from_post(productslug, formdata):
    product = Product.objects.get_by_site(slug=productslug)
    log.debug('found product: %s', product)
    p_types = product.get_subtypes()
    details = []
    zero = Decimal("0.00")

    if 'ConfigurableProduct' in p_types:
        # This happens when productname cannot be updated by javascript.
        cp = product.configurableproduct
        chosenOptions = optionids_from_post(cp, formdata)
        optproduct = cp.get_product_from_options(chosenOptions)
        if not optproduct:
            log.debug('Could not fully configure a ConfigurableProduct [%s] with [%s]', product, chosenOptions)
            raise Product.DoesNotExist()
        else:
            product = optproduct

    if 'CustomProduct' in p_types:
        cp = product.customproduct
        for customfield in cp.custom_text_fields.all():
            if customfield.price_change is not None:
                price_change = customfield.price_change
            else:
                price_change = zero
            data = { 'name' : customfield.translated_name(),
                     'value' : formdata["custom_%s" % customfield.slug],
                     'sort_order': customfield.sort_order,
                     'price_change': price_change }
            details.append(data)
            data = {}
        chosenOptions = optionids_from_post(cp, formdata)
        manager = OptionManager()
        for choice in chosenOptions:
            result = manager.from_unique_id(choice)
            if result.price_change is not None:
                price_change = result.price_change
            else:
                price_change = zero
            data = { 'name': unicode(result.option_group),
                      'value': unicode(result.translated_name()),
                      'sort_order': result.sort_order,
                      'price_change': price_change
            }
            details.append(data)
            data = {}

    if 'GiftCertificateProduct' in p_types:
        ix = 0
        for field in ('email', 'message'):
            data = {
                'name' : field,
                'value' : formdata.get("custom_%s" % field, ""),
                'sort_order' : ix,
                'price_change' : zero,
            }
            ix += 1
            details.append(data)
        log.debug("Gift Certificate details: %s", details)
        data = {}

    return product, details

def _product_error(request, product, msg):
    request.session['ERRORS'] = msg
    log.debug('Product Error: %s', msg)
    
    if request.is_ajax():
        return _json_response({'errors': [msg,]}, error=True)
    else:
        request.session['ERRORS'] = msg
        return HttpResponseRedirect(product.get_absolute_url())




from decimal import Decimal
from django import db
from django.conf import settings
from django.contrib.sites.models import Site
from django.core import urlresolvers
from django.db.models import Model
from django.http import HttpResponse
from django.test import TestCase
from product.forms import ProductExportForm
from product.models import *
from product.models import Category, ConfigurableProduct, Discount, ProductVariation, Option, OptionGroup, Product, Price
from product.utils import serialize_options, productvariation_details
from StringIO import StringIO
import datetime
import keyedcache
import signals
import zipfile

class OptionGroupTest(TestCase):
    
    def setUp(self):
        self.site=Site.objects.get_current()
        sizes = OptionGroup.objects.create(name="sizes", sort_order=1, site=self.site)
        option_small = Option.objects.create(option_group=sizes, name="Small", value="small", sort_order=1)
        option_large = Option.objects.create(option_group=sizes, name="Large", value="large", sort_order=2, price_change=1)
        colors = OptionGroup.objects.create(name="colors", sort_order=2, site=self.site)
        option_black = Option.objects.create(option_group=colors, name="Black", value="black", sort_order=1)
        option_white = Option.objects.create(option_group=colors, name="White", value="white", sort_order=2, price_change=3)

        # Change an option
        option_white.price_change = 5
        option_white.sort_order = 2
        option_white.save()
        
        self.sizes = sizes
        self.option_small = option_small
        self.option_large = option_large
        self.colors = colors
        self.option_black = option_black
        self.option_white = option_white
        
    def testUniqueTogether(self):
        """You can't have two options with the same value in an option group"""
        self.option_white.value = "black"
        try:
            self.option_white.save()
            self.fail('Should have thrown an error, due to duplicate keys')
        except db.IntegrityError:
            pass
        db.transaction.rollback()
        
    def testValues(self):
        opt = Option.objects.get(id=self.option_white.id)
        self.assertEqual(opt.value, u'white')
        self.assertEqual(opt.price_change, 5)
        self.assertEqual(opt.sort_order, 2)   
        
    def testConfigurable(self):
        """Create a configurable product, testing ordering and price"""
        django_shirt = Product.objects.create(slug="django-shirt", name="Django shirt", site=self.site)
        shirt_price = Price.objects.create(product=django_shirt, price="10.5")
        django_config = ConfigurableProduct.objects.create(product=django_shirt)
        django_config.option_group.add(self.sizes, self.colors)
        ordering = django_config.option_group.order_by('name')
        self.assertEqual(ordering[0], self.colors)
        self.assertEqual(ordering[1], self.sizes)
        django_config.save()

        # Create a product variation
        white_shirt = Product.objects.create(slug="django-shirt_small_white",
            name="Django Shirt (White/Small)", site=self.site)
        pv_white = ProductVariation.objects.create(product=white_shirt, parent=django_config)
        pv_white.options.add(self.option_white, self.option_small)
        self.assertEqual(pv_white.unit_price, Decimal("15.50"))

    def testConfigurableSlugs(self):
        """Create a product with a slug that could conflict with an 
        automatically generated product's slug."""
        
        django_shirt = Product.objects.create(slug="django-shirt", name="Django shirt", site=self.site)
        shirt_price = Price.objects.create(product=django_shirt, price="10.5")
        django_config = ConfigurableProduct.objects.create(product=django_shirt)
        django_config.option_group.add(self.sizes, self.colors)
        django_config.save()
        
        # Create a product with a slug that could conflict with an automatically
        # generated product's slug.
        clash_shirt = Product.objects.create(slug="django-shirt_small_black",
            name="Django Shirt (Black/Small)", site=self.site)

        # Automatically create the rest of the product variations
        django_config.create_subs = True
        django_config.save()
        self.assertEqual(ProductVariation.objects.filter(parent=django_config).count(), 4)

class CategoryTest(TestCase):
    """
    Run some category tests on urls
    """

    def setUp(self):
        self.site = Site.objects.get_current()

    def tearDown(self):
        keyedcache.cache_delete()

#    def test_absolute_url(self):
#        pet_jewelry = Category.objects.create(slug="pet-jewelry", name="Pet Jewelry", site=self.site)
#        womens_jewelry = Category.objects.create(slug="womens-jewelry", name="Women's Jewelry", site=self.site)
#        pet_jewelry.parent = womens_jewelry
#        pet_jewelry.save()
#        womens_jewelry.parent = pet_jewelry
#        self.assertRaises(ValidationError, womens_jewelry.save)
#        Model.save(womens_jewelry)
#        womens_jewelry = Category.objects.active().get(slug="womens-jewelry")
#        url = urlresolvers.reverse('satchmo_category', None, {'parent_slugs', 'womens-jewelry'})
#        self.assertEqual(womens_jewelry.get_absolute_url(), url)

#    def test_infinite_loop(self):
#        """Check that Category methods still work on a Category whose parents list contains an infinite loop."""
#        # Create two Categories that are each other's parents. First make sure that
#        # attempting to save them throws an error, then force a save anyway.
#        pet_jewelry = Category.objects.create(slug="pet-jewelry", name="Pet Jewelry", site=self.site)
#        womens_jewelry = Category.objects.create(slug="womens-jewelry", name="Women's Jewelry", site=self.site)
#        pet_jewelry.parent = womens_jewelry
#        pet_jewelry.save()
#        womens_jewelry.parent = pet_jewelry
#        try:
#            womens_jewelry.save()
#            self.fail('Should have thrown a ValidationError')
#        except ValidationError:
#            pass

#        # force save
#        Model.save(womens_jewelry)
#        pet_jewelry = Category.objects.active().get(slug="pet-jewelry")
#        womens_jewelry = Category.objects.active().get(slug="womens-jewelry")

#        kids = Category.objects.by_site(site=self.site).order_by('name')
#        slugs = [cat.slug for cat in kids]
#        self.assertEqual(slugs, [u'pet-jewelry', u'womens-jewelry'])

class DiscountTest(TestCase):

    def setUp(self):
        self.site = Site.objects.get_current()
        start = datetime.date(2006, 10, 1)
        end = datetime.date(5000, 10, 1)
        self.discount = Discount.objects.create(description="New Sale", code="BUYME", amount="5.00", allowedUses=10,
            numUses=0, minOrder=5, active=True, startDate=start, endDate=end, shipping='NONE', site=self.site)
    
    def tearDown(self):
        keyedcache.cache_delete()
    
    def testValid(self):

        v = self.discount.isValid()
        self.assert_(v[0])
        self.assertEqual(v[1], u'Valid.')

    def testFutureDate(self):
        """Test a future date for discount start"""
        start = datetime.date(5000, 1, 1)
        self.discount.startDate = start
        self.discount.save()
        self.discount.isValid()
        v = self.discount.isValid()
        self.assertFalse(v[0])
        self.assertEqual(v[1], u'This coupon is not active yet.')

    def testPastDate(self):
        """Test an expired discount"""
        #Change end date to the past
        start = datetime.date(2000, 1, 1)
        end = datetime.date(2006, 1, 1)
        self.discount.startDate = start
        self.discount.endDate = end
        self.discount.save()
        v = self.discount.isValid()
        self.assertFalse(v[0])
        self.assertEqual(v[1], u'This coupon has expired.')

    def testNotActive(self):
        """Not active should always be invalid."""
        self.discount.startDate = datetime.date(2006, 12, 1)
        self.discount.endDate = datetime.date(5000, 12, 1)
        self.discount.active = False
        self.discount.save()
        v = self.discount.isValid()
        self.assertFalse(v[0], False)
        self.assertEqual(v[1], u'This coupon is disabled.')
                

class CalcFunctionTest(TestCase):
    
    def testEvenSplit1(self):
        """Simple split test"""
        d = {
            1 : Decimal("10.00"),
            2 : Decimal("10.00"),
            3 : Decimal("10.00"),
            4 : Decimal("10.00"),
        }
        
        s = Discount.apply_even_split(d, Decimal("16.00"))
        self.assertEqual(s[1], Decimal("4.00"))
        self.assertEqual(s[2], Decimal("4.00"))
        self.assertEqual(s[3], Decimal("4.00"))
        self.assertEqual(s[4], Decimal("4.00"))
        
    def testEvenSplitTooMuch(self):
        """Test when amount is greater than total"""
        d = {
            1 : Decimal("10.00"),
            2 : Decimal("10.00"),
            3 : Decimal("10.00"),
            4 : Decimal("10.00"),
        }
        
        s = Discount.apply_even_split(d, Decimal("50.00"))
        self.assertEqual(s[1], Decimal("10.00"))
        self.assertEqual(s[2], Decimal("10.00"))
        self.assertEqual(s[3], Decimal("10.00"))
        self.assertEqual(s[4], Decimal("10.00"))

    def testEvenSplitEqual(self):
        """Test when amount is exactly equal"""
        d = {
            1 : Decimal("10.00"),
            2 : Decimal("10.00"),
            3 : Decimal("10.00"),
            4 : Decimal("10.00"),
        }

        s = Discount.apply_even_split(d, Decimal("40.00"))
        self.assertEqual(s[1], Decimal("10.00"))
        self.assertEqual(s[2], Decimal("10.00"))
        self.assertEqual(s[3], Decimal("10.00"))
        self.assertEqual(s[4], Decimal("10.00"))
        
        
    def testEvenSplitOneTooSmall(self):
        """Test when one of the items is maxed, but others are OK"""
        d = {
            1 : Decimal("10.00"),
            2 : Decimal("5.00"),
            3 : Decimal("10.00"),
            4 : Decimal("10.00"),
        }

        s = Discount.apply_even_split(d, Decimal("23.00"))
        self.assertEqual(s[1], Decimal("6.00"))
        self.assertEqual(s[2], Decimal("5.00"))
        self.assertEqual(s[3], Decimal("6.00"))
        self.assertEqual(s[4], Decimal("6.00"))

    def testThirds(self):
        d = {
            1 : Decimal("10.00"),
            2 : Decimal("10.00"),
            3 : Decimal("10.00"),
        }

        s = Discount.apply_even_split(d, Decimal("10.00"))
        self.assertEqual(s[1], Decimal("3.33"))
        self.assertEqual(s[2], Decimal("3.33"))
        self.assertEqual(s[3], Decimal("3.33"))


    def testThirdsUneven(self):
        d = {
            1 : Decimal("10.00"),
            2 : Decimal("10.00"),
            3 : Decimal("3.00"),
        }

        s = Discount.apply_even_split(d, Decimal("10.00"))
        self.assertEqual(s[1], Decimal("3.50"))
        self.assertEqual(s[2], Decimal("3.50"))
        self.assertEqual(s[3], Decimal("3.00"))


    def testPercentage1(self):
        d = {
            1 : Decimal("10.00"),
            2 : Decimal("10.00"),
            3 : Decimal("10.00"),
        }

        s = Discount.apply_percentage(d, Decimal("10.00"))
        self.assertEqual(s[1], Decimal("1.00"))
        self.assertEqual(s[2], Decimal("1.00"))
        self.assertEqual(s[3], Decimal("1.00"))


class ProductExportTest(TestCase):
    """
    Test product export functionality.
    """

    def setUp(self):
        # Log in as a superuser
        from django.contrib.auth.models import User
        user = User.objects.create_user('root', 'root@eruditorum.com', '12345')
        user.is_staff = True
        user.is_superuser = True
        user.save()
        self.client.login(username='root', password='12345')
        
    def tearDown(self):
        keyedcache.cache_delete()

    def test_text_export(self):
        """
        Test the content type of an exported text file.
        """
        url = urlresolvers.reverse('satchmo_admin_product_export')

        form_data = {
            'format': 'yaml',
            'include_images': False,
        }

        response = self.client.post(url, form_data)
        self.assertTrue(response.has_header('Content-Type'))
        self.assertEqual('text/yaml', response['Content-Type'])

        form_data['format'] = 'json'
        response = self.client.post(url, form_data)
        self.assertTrue(response.has_header('Content-Type'))
        self.assertEqual('text/json', response['Content-Type'])

        form_data['format'] = 'xml'
        response = self.client.post(url, form_data)
        self.assertTrue(response.has_header('Content-Type'))
        self.assertEqual('text/xml', response['Content-Type'])

        form_data['format'] = 'python'
        response = self.client.post(url, form_data)
        self.assertTrue(response.has_header('Content-Type'))
        self.assertEqual('text/python', response['Content-Type'])

    def test_zip_export_content_type(self):
        """
        Test the content type of an exported zip file.
        """
        url = urlresolvers.reverse('satchmo_admin_product_export')
        form_data = {
            'format': 'yaml',
            'include_images': True,
        }

        response = self.client.post(url, form_data)
        self.assertTrue(response.has_header('Content-Type'))
        self.assertEqual('application/zip', response['Content-Type'])
        
    def test_unicode(self):
        """Test the ProductExportForm behavior
        Specifically, we're checking that a unicode 'format' is converted to ascii
        in the 'export' method of 'ProductExportForm'."""
        form = ProductExportForm({'format': u'yaml', 'include_images': True})
        response = form.export(None)
        self.assert_(isinstance(response, HttpResponse))

class ProductTest(TestCase):
    """Test Product functions"""
    fixtures = ['l10n-data.yaml','sample-store-data.yaml', 'products.yaml', 'test-config.yaml']

    def tearDown(self):
        keyedcache.cache_delete()

    def test_quantity_price_standard_product(self):
        """Check quantity price for a standard product"""

        product = Product.objects.get(slug='PY-Rocks')
        self.assertEqual(product.unit_price, Decimal("19.50"))

    def test_discount_qty_price(self):
        """Test quantity price discounts"""
        product = Product.objects.get(slug='PY-Rocks')
        price = Price(product=product, quantity=Decimal('10'), price=Decimal("10.00"))
        price.save()

        self.assertEqual(product.unit_price, Decimal("19.50"))
        self.assertEqual(product.get_qty_price(Decimal('1')), Decimal("19.50"))
        self.assertEqual(product.get_qty_price(Decimal('2')), Decimal("19.50"))
        self.assertEqual(product.get_qty_price(Decimal('10')), Decimal("10.00"))
        
    def test_expiring_price(self):
        """Test whether a price with an expiration date is used in preference to a non-expiring price."""
        product = Product.objects.get(slug='PY-Rocks')
        self.assertEqual(product.unit_price, Decimal("19.50"))

        today = datetime.datetime.now()
        aweek = datetime.timedelta(days=7)
        lastwk = today - aweek
        nextwk = today + aweek

        # new price should override the old one
        price = Price.objects.create(product=product, quantity=Decimal('1'), price=Decimal("10.00"), expires=nextwk)
        self.assertEqual(product.unit_price, Decimal("10.00"))

        # but not if it is expired
        price.expires = lastwk
        price.save()
        self.assertEqual(product.unit_price, Decimal("19.50"))

    def test_quantity_price_productvariation(self):
        """Check quantity price for a productvariation"""

        # base product
        product = Product.objects.get(slug='dj-rocks')
        self.assertEqual(product.unit_price, Decimal("20.00"))
        self.assertEqual(product.unit_price, product.get_qty_price(Decimal('1')))

        # product with no price delta
        product = Product.objects.get(slug='dj-rocks-s-b')
        self.assertEqual(product.unit_price, Decimal("20.00"))
        self.assertEqual(product.unit_price, product.get_qty_price(Decimal('1')))

        # product which costs more due to details
        product = Product.objects.get(slug='dj-rocks-l-bl')
        self.assertEqual(product.unit_price, Decimal("23.00"))
        self.assertEqual(product.unit_price, product.get_qty_price(Decimal('1')))


    def test_quantity_price_productvariation_expiring(self):
        """Check expiring quantity price for a productvariation"""

        # base product
        product = Product.objects.get(slug='dj-rocks')
        self.assertEqual(product.unit_price, Decimal("20.00"))
        self.assertEqual(product.unit_price, product.get_qty_price(Decimal('1')))

        today = datetime.datetime.now()
        aweek = datetime.timedelta(days=7)
        nextwk = today + aweek

        # new price should override the old one
        price = Price.objects.create(product=product, quantity=Decimal('1'), price=Decimal("10.00"), expires=nextwk)
        self.assertEqual(product.unit_price, Decimal("10.00"))

        # product with no price delta
        product = Product.objects.get(slug='dj-rocks-s-b')
        self.assertEqual(product.unit_price, Decimal("10.00"))

        # product which costs more due to details
        product = Product.objects.get(slug='dj-rocks-l-bl')
        self.assertEqual(product.unit_price, Decimal("13.00"))

        # now test explicit expiring price on a product variation
        pvprice = Price.objects.create(product=product, quantity=Decimal('1'), price=Decimal("5.00"), expires=nextwk)
        self.assertEqual(product.unit_price, Decimal("5.00"))

    def test_smart_attr(self):
        p = Product.objects.get(slug__iexact='dj-rocks')
        mb = Product.objects.get(slug__iexact='dj-rocks-m-b')
        sb = Product.objects.get(slug__iexact='dj-rocks-s-b')

        # going to set a weight on the product, and an override weight on the medium
        # shirt.

        p.weight = 100
        p.save()
        sb.weight = 50
        sb.save()

        self.assertEqual(p.smart_attr('weight'), 100)
        self.assertEqual(sb.smart_attr('weight'), 50)
        self.assertEqual(mb.smart_attr('weight'), 100)

        # no height
        self.assertEqual(p.smart_attr('height'), None)
        self.assertEqual(sb.smart_attr('height'), None)

class ConfigurableProductTest(TestCase):
    """Test ConfigurableProduct."""
    fixtures = ['products.yaml']
    
    def tearDown(self):
        keyedcache.cache_delete()

    def test_get_variations_for_options(self):
        # Retrieve the objects.
        dj_rocks = ConfigurableProduct.objects.get(product__slug="dj-rocks")
        option_small = Option.objects.get(pk=1)
        option_black = Option.objects.get(pk=4)
        option_hard_cover = Option.objects.get(pk=7)

        # Test filtering for one valid option.
        self.assertEqual([variation.pk for variation in
            dj_rocks.get_variations_for_options([option_small])], [6, 7, 8])
        # Test filtering for two valid options.
        self.assertEqual([variation.pk for variation in
            dj_rocks.get_variations_for_options([option_small, option_black])],
            [6])
        # Test filtering for an option that cannot apply to the product.
        self.assertEqual(
            len(dj_rocks.get_variations_for_options([option_hard_cover])), 0)
        # Test filtering for nothing.
        self.assertEqual([variation.pk for variation in
            dj_rocks.get_variations_for_options([])],
            [6, 7, 8, 9, 10, 11, 12, 13, 14])

class OptionUtilsTest(TestCase):
    """Test the utilities used for serialization of options and selected option details."""
    fixtures = ['products.yaml']
    
    def test_base_sort_order(self):
        p = Product.objects.get(slug='dj-rocks')
        serialized = serialize_options(p.configurableproduct)
        self.assert_(len(serialized), 2)
        self.assertEqual(serialized[0]['id'], 1)
        got_vals = [opt.value for opt in serialized[0]['items']]
        self.assertEqual(got_vals, ['S','M','L'])
        self.assertEqual(serialized[1]['id'], 2)

    def test_reordered(self):
        p = Product.objects.get(slug='dj-rocks')
        
        pv = p.configurableproduct.productvariation_set.all()[0]
        orig_key = pv.optionkey
        orig_detl = productvariation_details(p, False, None, create=True)
        
        sizegroup = OptionGroup.objects.get(name="sizes")
        sizegroup.sort_order = 100
        sizegroup.save()
        
        # reverse ordering
        for opt in sizegroup.option_set.all():
            opt.sort_order = 100-opt.sort_order
            opt.save()
        
        serialized = serialize_options(p.configurableproduct)
        self.assert_(len(serialized), 2)
        self.assertEqual(serialized[1]['id'], 1)
        got_vals = [opt.value for opt in serialized[1]['items']]
        self.assertEqual(got_vals, ['L','M','S'])
        
        pv2 = ProductVariation.objects.get(pk=pv.pk)
        self.assertEqual(orig_key, pv2.optionkey)
        reorder_detl = productvariation_details(p, False, None)
        self.assertEqual(orig_detl, reorder_detl)

class PriceAdjustmentTest(TestCase):
    fixtures = ['products.yaml']
    
    def setUp(self):
        self.product = Product.objects.get(slug="dj-rocks")
        self.price = self.product.price_set.get(quantity=1)
        
    def tearDown(self):
        keyedcache.cache_delete()
            
    def test_basic(self):
        pcalc = PriceAdjustmentCalc(self.price)
        p = PriceAdjustment('test', amount=Decimal(1))
        pcalc += p
        pcalc += p
        p = PriceAdjustment('test2', amount=Decimal(10))
        pcalc += p
        self.assertEqual(pcalc.total_adjustment(), Decimal(12))
        
    def test_product_adjustments(self):
        p1 = self.product.unit_price
        self.assertEqual(p1, Decimal('20.00'))
        signals.satchmo_price_query.connect(five_off)
        p2 = self.product.unit_price
        self.assertEqual(p2, Decimal('15.00'))
        adj = get_product_quantity_adjustments(self.product, qty=1)
        self.assertEqual(len(adj.adjustments), 1)
        a = adj.adjustments[0]
        self.assertEqual(a.key, 'half')
        self.assertEqual(a.amount, Decimal(5))
        signals.satchmo_price_query.disconnect(five_off)

def five_off(sender, adjustment=None, **kwargs):
    adjustment += PriceAdjustment('half', 'Half', amount=Decimal(5))

if __name__ == "__main__":
    import doctest
    doctest.testmod()




import base64
import hmac
import hashlib
import urllib
import unittest

def createhash(key, value):
    hash = hmac.new(key, value).hexdigest() 
    return base64.urlsafe_b64encode(hash)

def test_createhash():
    key = '1234'
    msg = '12'

    #base64 encoded data (the url)
    hash = createhash(key, msg)
    hash2 = hmac.new(key, base64.urlsafe_b64decode(hash)).hexdigest()
    
    assert hash == hash2, 'hmac keys not equal!'

if __name__ == '__main__':
    test_createhash()




from django.db import models
from django.utils.translation import ugettext_lazy as _
from satchmo_store.contact.models import Contact
import datetime
import logging

log = logging.getLogger('newsletter.models')

_NOTSET = object()

class NullContact(object):
    """Simple object emulating a Contact, so that we can add users who aren't Satchmo Contacts.

    Note, this is *not* a Django object, and is not saved to the DB, only to the subscription lists.
    """

    def __init__(self, full_name, email):
        if not full_name:
            full_name = email.split('@')[0]

        self.full_name = full_name
        self.email = email

def get_contact_or_fake(full_name, email):
    """Get a `Contact` by email or if it doesn't exist, then a `NullContact`"""
    try:
        contact = Contact.objects.get(email=email)

    except Contact.DoesNotExist:
        contact = NullContact(full_name, email)

    return contact

class Subscription(models.Model):
    """A newsletter subscription."""

    subscribed = models.BooleanField(_('Subscribed'), default=True)
    email = models.EmailField(_('Email'), max_length=75)
    create_date = models.DateField(_("Creation Date"))
    update_date = models.DateField(_("Update Date"))

    def email_is_subscribed(cls, email):
        try:
            sub = cls.objects.get(email=email)
            return sub.subscribed
        except cls.DoesNotExist:
            return False

    email_is_subscribed = classmethod(email_is_subscribed)

    def __unicode__(self):
        if self.subscribed:
            flag="Y"
        else:
            flag="N"
        return u"[%s] %s" % (flag, self.email)

    def __repr__(self):
        return "<Subscription: %s>" % str(self)

    def attribute_value(self, name, value=_NOTSET):
        """Get a value from an attribute."""
        try:
            att = self.attributes.get(name=name)
            value = att.value
        except SubscriptionAttribute.DoesNotExist:
            if value != _NOTSET:
                raise

        return value

    def save(self, **kwargs):
        if not self.pk:
            self.create_date = datetime.date.today()

        self.update_date = datetime.date.today()

        super(Subscription, self).save(**kwargs)
                
    def update_attribute(self, name, value):
        """Update or create a `SubscriptionAttribute` object with the passed `name` and `value`."""
        value = str(value)
        try:
            att = self.attributes.get(name=name)
            att.value = value
        except SubscriptionAttribute.DoesNotExist:
            att = SubscriptionAttribute(subscription=self, name=name, value=value)
        
        att.save()
        return att

    def update_attributes(self, attributes):
        """Update `SubscriptionAttribute` objects from a dictionary of name val mappings."""
        return [self.update_attribute(name, value) for name, value in attributes.items()]

        
class SubscriptionAttribute(models.Model):
    """
    Allows arbitrary name/value pairs (as strings) to be attached to a subscription.
    """
    subscription = models.ForeignKey(Subscription, related_name="attributes")
    name = models.SlugField(_("Attribute Name"), max_length=100, )
    value = models.CharField(_("Value"), max_length=255)

    class Meta:
        verbose_name = _("Subscription Attribute")
        verbose_name_plural = _("Subscription Attributes")

import config
import listeners
listeners.start_listening()



from django.utils.translation import ugettext_lazy as _
from livesettings import *
from product.config import PRODUCT_GROUP

BRAND_SLUG = config_register(
    StringValue(PRODUCT_GROUP,
        'BRAND_SLUG',
        description=_("Brand Slug"),
        help_text=_("The url slug for brands.  Requires server restart if changed."),
        default="brand"))



"""TestCollection API

A bunch of functions and classes to extend on the ``unittest.TestLoader`` 
class for loading custom test collections. 

Test collections are regular directories to search for test scripts ::

    [app:myapp]
    collections = "fruits,vegetable,algae,mango"

"""

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import os
import posixpath
import re
import sys

from notmm.utils.cutandpaste import colored as _

__all__ = (
    'get_base_dir',
    'get_test_modules',
    'is_test_script',
    'modfunc_noop',
    'test_script_re', 
    'TestCollection',
    'CollectionError'
    )

test_script_re = re.compile(r"test_\w+\.py$")

#logger = logging.getLogger(__name__)
#def _log(message):
#    logger.debug(message)    

# shortcut function to check if ``s`` is present in sys.path.
modfunc_noop = lambda s: bool(s in sys.modules)

_listoperator = lambda l: [item.strip() for item in l.rsplit(',')] 
_joinoperator = lambda a,b: posixpath.join(a, b)

class CollectionError(Exception):
    """An exception occured processing a test collection"""


def get_base_dir(dirname, _homedir='tests'):
    """Helper function to return a common prefix for test 
    scripts within the same directory structure.

    ``dirname``: Path to a directory holding test scripts.
    
    >>> get_base_dir('.')
    '.'
    >>> get_base_dir('dbapi/orm/elixir')
    'dbapi/orm/elixir'

    """
    #print dirname
    parts = dirname.rsplit('/', 1)
    if len(parts) >= 2:
        # return the last two components
        return os.path.join(parts[0], parts[-1])
    return parts[-1]


def is_test_script(pathname):
    """Determine if ``pathname`` is a test module and return True."""
    # For now, if its starting with prefix and
    # its ending with '.py' then return True.
    if isinstance(pathname, basestring) \
    and re.search(test_script_re, pathname):
        return True
    return False


def get_test_modules(collection, _topdown=False): 
    """Returns a sequence of usable test files (iterator)."""

    l = list()

    for item in _listoperator(collection):
        path = posixpath.realpath(item)
        if not os.path.exists(path):
            raise CollectionError("inexistent path: %r" % path)
        for dirname, names, files in os.walk(item, topdown=_topdown):
            basedir = get_base_dir(dirname)
            l.extend((# iterator start
                (_joinoperator(basedir, f), basedir) for f in files if is_test_script(f) 
                # iterator end
                ))
    return frozenset(l)

class TestCollection(unittest.TestLoader):
    """Helper class extending ``unittest.TestLoader``. 
    
    XXX: Put more documentation here.
    """
    
    def __init__(self, quiet=True, debug=0):
        super(TestCollection, self).__init__()
        self.quiet = quiet
        self.suite = unittest.TestSuite()
        self.interactive = False
        self.debug = debug

    def findTestCases(self, obj, 
        interactive=False, _TestCaseClass=unittest.TestCase):
        """
        Helper function to collect test cases inside a Python object
        or module. 
        """
        l = set()
        for item, value in obj.__dict__.items():
            if (item.endswith('TestCase') and \
            issubclass(value, _TestCaseClass)):
                # XXX Only attempt to filter test cases which belongs
                # to the test script
                try:
                    if interactive:
                        import pdb; pdb.set_trace()
                    testSuite = self.loadTestsFromTestCase(value)
                    #testCase = super(_TestCaseClass, value).__new__(*value.__bases__)
                    #_TestCaseClass.__init__(testCase)
                finally:
                    for testMethod in testSuite._tests:
                        #print "DEBUG: appending test method to test suite: %s" % testMethod
                        l.add(testMethod)
            #else:
            #    #print "ignored %s" % item
        
        return frozenset(l)

    def loadTestsFromNames(self, names, keep_going=False, interactive=False):
        """ Return a ``unittest.TestSuite`` object.

        This method can be used to create a custom test suite from 
        a list of user-defined test directories ::  

        .. coding: python

            >>> loader = TestCollection()
            >>> s2 = loader.loadTestsFromNames('apple,strawberry,pear')
        """
        tests = []
        for name in names.split(','):
            #relname = os.path.basename(name)[:-3] # remove the extension
            if not self.quiet: 
                _('Loading base=%r ...' % name)
            tests.append(
                self.discover(
                    os.path.realpath(name), top_level_dir=os.getcwd())
                    )
        self.suite.addTests(tests)
        #    self.suite.addTests(
        #        self.findTestCases(name, interactive=interactive)
        #        )
        #if self.debug > 1:
        #    print "[Testcases found: %d]" % self.suite.countTestCases()

        # Allow interactive debugging session if IPython is present
        #if len(resultset) >= 1:
        #    if not self.quiet:
        #        _("Found %d tests contained in %d modules." % (len(items), \
        #            len(names)))
        #        _("Total test cases count: %d." % suite.countTestCases())
        return self.suite

    loadTestsFromList = loadTestsFromNames
    



# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from gazpacho import gapi, util
from gazpacho.clipboard import clipboard
from gazpacho.command import ContainerCommand
from gazpacho.commandmanager import command_manager
from gazpacho.properties import CommandSetProperty
from gazpacho.i18n import _

class Popup(object):
    """This class defines a popup menu and its behaviour

    This popup menu has entries for selecting the widget, cut/copy/paste it
    and deleting it.

    It also shows a submenu for every parent widget in the hierarchy.
    """
    def __init__(self, command_manager, gadget):
        self._command_manager = command_manager
        self._gadget = gadget
        self._popup_menu = self._create_menu(gadget)

    def pop(self, event):
        """Call this method if you want the menu to show up"""
        if event:
            button = event.button
            event_time = event.time
        else:
            button = 0
            event_time = 0

        self._popup_menu.popup(None, None, None, button, event_time)
        return self._popup_menu

    # callbacks for the actions
    def _select_cb(self, item, gadget):
        if gadget:
            gadget.select()

    def _cut_cb(self, item, gadget):
        clipboard.cut(gadget)

    def _copy_cb(self, item, gadget):
        clipboard.copy(gadget)

    def _paste_cb(self, item, placeholder):
        gadget = util.get_parent(placeholder)
        if isinstance(placeholder, gtk.TreeView):
            from gazpacho.gadget import Gadget
            clipboard.paste(Gadget.from_widget(placeholder))
        else:
            clipboard.paste(placeholder, gadget.project)

    def _delete_cb(self, item, gadget):
        gapi.delete_gadget(gadget.project, gadget)

    def _box_insert_cb(self, item, gadget, after):
        from gazpacho.widgets.base.box import CommandBoxInsertPlaceholder
        parent_pos = self._find_parent_box_and_pos(gadget)
        if parent_pos is None:
            return

        (parent, pos) = parent_pos
        cmd = CommandBoxInsertPlaceholder(parent, pos, after)
        self._command_manager.execute(cmd, parent.project)

    def _table_insert(self, gadget, row=False, column=False, before=False):
        from gazpacho.gadget import Gadget

        selected = gadget.project.selection[0]
        table = gadget.widget

        # First resize table
        if row:
            old_prop_name = 'top-attach'
            n_rows = gadget.get_prop('n-rows')
            cmds = [CommandSetProperty(n_rows, n_rows.value + 1)]
            desc = 'row'
        elif column:
            old_prop_name = 'left-attach'
            n_columns = gadget.get_prop('n-columns')
            cmds = [CommandSetProperty(n_columns, n_columns.value + 1)]
            desc = 'column'
        else:
            raise AssertionError

        # XXX: Remove this hack, it is selection related, the table
        #      might be selected when we enter here. Requires UI changes
        try:
            old = table.child_get_property(selected, old_prop_name)
        except TypeError:
            return

        if before:
            old -= 1

        # Secondly move children
        for child in table.get_children():
            child_gadget = Gadget.from_widget(child)
            # No need to move placeholders
            if not child_gadget:
                continue

            if row:
                pos = child_gadget.get_child_prop('y-pos')
            else: # column
                pos = child_gadget.get_child_prop('x-pos')

            if pos.value > old:
                cmds.append(CommandSetProperty(pos, pos.value + 1))

        # Execute everything in one go
        command_manager.execute(
            ContainerCommand('Adding a %s to table' % desc, *cmds),
            gadget.project)

    def _table_insert_row_before_cb(self, item, gadget):
        self._table_insert(gadget, row=True, before=True)

    def _table_insert_row_after_cb(self, item, gadget):
        self._table_insert(gadget, row=True)

    def _table_insert_column_before_cb(self, item, gadget):
        self._table_insert(gadget, column=True, before=True)

    def _table_insert_column_after_cb(self, item, gadget):
        self._table_insert(gadget, column=True)

    # aux methods
    def _create_menu(self, gadget, add_children=True):
        from gazpacho.placeholder import Placeholder
        popup_menu = gtk.Menu()

        self._append_item(popup_menu, None,
                          _('Select')+' '+gadget.name,
                          True,
                          self._select_cb, gadget)
        self._append_item(popup_menu, gtk.STOCK_CUT, None, True,
                          self._cut_cb, gadget)
        self._append_item(popup_menu, gtk.STOCK_COPY, None, True,
                          self._copy_cb, gadget)
        if isinstance(gadget, Placeholder):
            self._append_item(popup_menu, gtk.STOCK_PASTE, None, True,
                              self._paste_cb, gadget)
        self._append_item(popup_menu, gtk.STOCK_DELETE, None, True,
                          self._delete_cb, gadget)

        if isinstance(gadget.widget, gtk.Box):
            separator = gtk.MenuItem()
            separator.show()
            popup_menu.append(separator)

            self._append_item(popup_menu, _('Insert before'), None, True,
                              self._box_insert_cb, gadget, False)

            self._append_item(popup_menu, _('Insert after'), None, True,
                              self._box_insert_cb, gadget, True)
        elif isinstance(gadget.widget, gtk.Table):
            self._append_item(popup_menu, _('Insert row before'), None, True,
                              self._table_insert_row_before_cb, gadget)
            self._append_item(popup_menu, _('Insert row after '), None, True,
                              self._table_insert_row_after_cb, gadget)
            self._append_item(popup_menu, _('Insert column before'), None, True,
                              self._table_insert_column_before_cb, gadget)
            self._append_item(popup_menu, _('Insert column after '), None, True,
                              self._table_insert_column_after_cb, gadget)

        if add_children and not gadget.is_toplevel():
            parent = gadget.get_parent()
            self._populate_children(popup_menu, parent)

        return popup_menu

    def _append_item(self, menu, stock, label, sensitive, callback, *data):
        if stock:
            menu_item = gtk.ImageMenuItem(stock, None)
        else:
            menu_item = gtk.MenuItem(label, False)

        menu_item.connect('activate', callback, *data)

        menu_item.set_sensitive(sensitive)
        menu_item.show()
        menu.append(menu_item)

    def _populate_children(self, popup_menu, parent):
        while parent:
            separator = gtk.MenuItem()
            separator.show()
            popup_menu.append(separator)

            child = gtk.MenuItem(parent.name, False)
            child_menu = self._create_child_menu(parent)
            child.set_submenu(child_menu)
            child.show()
            popup_menu.append(child)

            parent = parent.get_parent()

    def _create_child_menu(self, parent):
        return Popup._create_menu(self, parent, False)

    # Used for finding the parent box to insert placeholder in
    def _find_parent_box_and_pos(self, gadget):
        from gazpacho import placeholder
        if isinstance(self._gadget, placeholder.Placeholder):
            child_widget = self._gadget
            parent = util.get_parent(self._gadget)
        else:
            child_widget = self._gadget.widget
            parent = self._gadget.get_parent()

        while parent:
            if parent == gadget:
                break

            child_widget = parent.widget
            parent = parent.get_parent()

        if parent is not None and child_widget is not None:
            pos = parent.widget.get_children().index(child_widget)

            return parent, pos


class PlaceholderPopup(Popup):
    def _delete_placeholder_cb(self, item, placeholder):
        from gazpacho.widgets.base.box import CommandBoxDeletePlaceholder

        parent = util.get_parent(placeholder)
        if len(parent.widget.get_children()) >= 1:
            cmd = CommandBoxDeletePlaceholder(placeholder)
            self._command_manager.execute(cmd, parent.project)

    def _create_menu(self, placeholder, add_children=False):
        from gazpacho.clipboard import clipboard
        popup_menu = gtk.Menu()

        # Paste menu item
        item = clipboard.get_selected_item()
        enable_paste = False
        if item and not item.is_toplevel:
            enable_paste = True
        self._append_item(popup_menu, gtk.STOCK_PASTE, None, enable_paste,
                          self._paste_cb, placeholder)

        # Delete menu item
        self._append_item(popup_menu, gtk.STOCK_DELETE, None,
                          placeholder.is_deletable(),
                          self._delete_placeholder_cb, placeholder)

        parent = util.get_parent(placeholder)
        self._populate_children(popup_menu, parent)

        return popup_menu




from django.utils.translation import ugettext_lazy as _
from livesettings import *
from tax.config import TAX_MODULE

TAX_MODULE.add_choice(('tax.modules.percent', _('Percent Tax')))
TAX_GROUP = config_get_group('TAX')

config_register(
    DecimalValue(TAX_GROUP,
        'PERCENT',
        description=_("Percent tax"),
        requires=TAX_MODULE,
        requiresvalue='tax.modules.percent')
)

config_register(
    BooleanValue(TAX_GROUP,
        'TAX_SHIPPING',
        description=_("Tax Shipping?"),
        requires=TAX_MODULE,
        requiresvalue='tax.modules.percent',
        default=False)
)



from django.core.management.base import BaseCommand
import os
from datetime import date

# We don't actually need it, but otherwise livesettings chokes.
import tax.config

from tax.modules.us_sst.models import TaxRate, TaxBoundry
from decimal import Decimal

def ash_split(arg, qty):
    """Unfortunately, states don't alwys publish the full SST fields in the
    boundry files like they are required to. It's a shame really. So this function
    will force a string to split to 'qty' fields, adding None values as needed to
    get there.
    """
    l = arg.split(',')
    if len(l) < qty:
        l.extend([None for x in xrange(qty-len(l))])
    return l

CSV_MAP = (
    'recordType', 'startDate', 'endDate',
    'lowAddress', 'highAddress', 'oddEven',
    'streetPreDirection', 'streetName', 'streetSuffix', 'streetPostDirection',
    'addressSecondaryAbbr', 'addressSecondaryLow', 'addressSecondaryHigh', 'addressSecondaryOddEven',
    'cityName', 'zipCode', 'plus4', 
    'zipCodeLow', 'zipExtensionLow', 'zipCodeHigh', 'zipExtensionHigh',
    'serCode',
    'fipsStateCode', 'fipsStateIndicator', 'fipsCountyCode', 'fipsPlaceCode', 'fipsPlaceType',
    'long', 'lat',
    'special_1_source', 'special_1_code', 'special_1_type',
    'special_2_source', 'special_2_code', 'special_2_type',
    'special_3_source', 'special_3_code', 'special_3_type',
    'special_4_source', 'special_4_code', 'special_4_type',
    'special_5_source', 'special_5_code', 'special_5_type',
    'special_6_source', 'special_6_code', 'special_6_type',
    'special_7_source', 'special_7_code', 'special_7_type',
    'special_8_source', 'special_8_code', 'special_8_type',
    'special_9_source', 'special_9_code', 'special_9_type',
    'special_10_source', 'special_10_code', 'special_10_type',
    'special_11_source', 'special_11_code', 'special_11_type',
    'special_12_source', 'special_12_code', 'special_12_type',
    'special_13_source', 'special_13_code', 'special_13_type',
    'special_14_source', 'special_14_code', 'special_14_type',
    'special_15_source', 'special_15_code', 'special_15_type',
    'special_16_source', 'special_16_code', 'special_16_type',
    'special_17_source', 'special_17_code', 'special_17_type',
    'special_18_source', 'special_18_code', 'special_18_type',
    'special_19_source', 'special_19_code', 'special_19_type',
    'special_20_source', 'special_20_code', 'special_20_type',
)
# Some fields we're not using.
DELETE_FIELDS = (
    'long', 'lat',
    'special_1_source',
    'special_2_source',
    'special_3_source',
    'special_4_source',
    'special_5_source',
    'special_6_source',
    'special_7_source',
    'special_8_source',
    'special_9_source',
    'special_10_source',
    'special_11_source',
    'special_12_source',
    'special_13_source',
    'special_14_source',
    'special_15_source',
    'special_16_source',
    'special_17_source',
    'special_18_source',
    'special_19_source',
    'special_20_source',
)
class Command(BaseCommand):
    '''Manage command to import one of the CSV files from the SST website.

    To update: Simple re-run on the newer CSV file.
    Any unchanged entries will be left alone, and any changed ones will get
    their end dates set properly and the new rows inserted. You will need to do
    this quartly or as-needed by your tax jurisdictions.'''


    def handle(self, *args, **options):
        new = 0
        updated = 0
        unchanged = 0
        total = 0
        file = args[0]
        if not os.path.isfile(file):
            raise RuntimeError("File: %s is not a normal file or doesn't exist." % file)
        file = open(file)
        print "Processing: ",
        for line in file:
            line = line.strip()
            #Z,20080701,99991231,,,,,,,,,,,,,,,00073,,00073,,EXTRA
            fields = ash_split(line, len(CSV_MAP))

            # Turn it all into a dict so we can search for a duplicate,
            # Then remove the keys we don't care about or use.
            d = dict([(x, fields.pop(0)) for x in CSV_MAP])
            for v in DELETE_FIELDS:
                del(d[v])

            d['recordType'] = d['recordType'].upper()
            d['startDate'] = date(int(d['startDate'][0:4]), int(d['startDate'][4:6]), int(d['startDate'][6:9]))
            d['endDate'] = date(int(d['endDate'][0:4]), int(d['endDate'][4:6]), int(d['endDate'][6:9]))

            # Empty strings are nulls.
            for k in d.keys():
                if d[k] == '':
                    d[k] = None

            if d['recordType'] == 'A':
                # For now, skip these, as they barely work.
                # Zip+4 is the best way always. These are a bad idea in general.
                continue

                d['lowAddress'] = int(d['lowAddress'])
                d['highAddress'] = int(d['highAddress'])
                d['oddEven'] = d['oddEven'].upper()
                d['addressSecondaryOddEven'] = d['addressSecondaryOddEven'].upper()
                d['zipCode']          = int(d['zipCode'])
                d['plus4']            = int(d['plus4'])
            elif d['recordType'] == '4':
                d['zipCodeLow']       = int(d['zipCodeLow'])
                d['zipExtensionLow']  = int(d['zipExtensionLow'])
                d['zipCodeHigh']      = int(d['zipCodeHigh'])
                d['zipExtensionHigh'] = int(d['zipExtensionHigh'])
            elif d['recordType'] == 'Z':
                d['zipCodeLow']       = int(d['zipCodeLow'])
                d['zipCodeHigh']      = int(d['zipCodeHigh'])

            end = d['endDate']
            del(d['endDate'])

            try:
                tb = TaxBoundry.objects.get(**d)
                # Over time, end dates can change. A new row with a new start
                # date will also appear. This way, loading a new file correctly
                # updates the map. (I hope.)
                if tb.endDate != end:
                    tb.endDate = end
                    tb.save()
                    total += 1
                    updated += 1
                else:
                    total += 1
                    unchanged += 1

            except TaxBoundry.DoesNotExist:
                # Put the end back, and save it.
                d['endDate'] = end
                try:
                    TaxBoundry(**d).save()
                    total += 1
                    new += 1
                except:
                    print "Error loading the following row:"
                    for k in CSV_MAP:
                        if k in d:
                            print "%s: '%s'" % (k, d[k])
                    raise

            if total % 100 == 0:
                print "%s," % total,
                
            # Now, handle mapping boundries to rates.
            #extra = SER,state_providing,state_taxed,County,Place,Class,Long,Lat, (ST/VD,Special Code,Special Type,) x 20
            # IF SER, then the tax module should report all sales taxes by that SER code.
            # Otherwise, report it by each applicable tax.
            # Total tax is still the same in both cases. Just the state wants it reported differently.
        print ""
        print "Done: New: %d. End date changed: %d. Unchanged: %d" % (new, updated, unchanged)



from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from livesettings import *
from satchmo_utils import is_string_like, load_module
from satchmo_store.shop import get_satchmo_setting

TAX_GROUP = ConfigurationGroup('TAX', _('Tax Settings'))
TAX_MODULE = config_register(StringValue(TAX_GROUP,
    'MODULE',
    description=_("Active tax module"),
    help_text=_("Select a module, save and reload to set any module-specific settings."),
    default="tax.modules.no",
    choices=[('tax.modules.no', _('No Tax')),
    ]
))

DEFAULT_VIEW_TAX = config_register(BooleanValue(TAX_GROUP,
    'DEFAULT_VIEW_TAX',
    description=_("Show with tax included"),
    help_text=_("If yes, then all products and the cart will display with tax included."),
    default=False
))

PRODUCTS_TAXABLE_BY_DEFAULT = config_register(BooleanValue(TAX_GROUP,
    'PRODUCTS_TAXABLE_BY_DEFAULT',
    description=_("New products are automatically made taxable"),
    help_text=_("Whether newly created products should be taxable by default."),
    default=False
))

TAX_AREA_ADDRESS = config_register(
        StringValue(TAX_GROUP,
        'TAX_AREA_ADDRESS',
        description=_("Should tax be calculated based on shipping or billing address?"),
        help_text=_("This will only be used if tax is calculated based on an address."),
        choices = (
            (('ship', 'Shipping')),
            (('bill', 'Billing')),
        ),
        default = 'ship')
        )

# --- Load any extra tax modules. ---
extra_tax = get_satchmo_setting('CUSTOM_TAX_MODULES')
for extra in extra_tax:
    try:
        load_module("%s.config" % extra)
    except ImportError:
        log.warn('Could not load tax module configuration: %s' % extra)



from django.utils.translation import ugettext_lazy as _
from livesettings import *
import logging
log = logging.getLogger('shipping.modules.productshipping')
from shipping.config import SHIPPING_ACTIVE

SHIPPING_ACTIVE.add_choice(('shipping.modules.productshipping', _('Shipping By Product')))

log.debug('loaded')



# Django Signals Ahoy
# Copyright Bruce Kroeze 2009
#
# Released under the "New BSD License"
# Please see LICENSE.rst in this distribution for license details.
# -----------------
import threading
import logging

log = logging.getLogger('signals_ahoy.asynchronous')

class AsynchronousListener(object):
    """A wrapper for signal listeners which will run the listen function in a separate thread."""
    
    def __init__(self, listenfunction, label="Anonymous"):
        self.listenfunction = listenfunction
        self.label = "label"
        
    def listen(self, *args, **kwargs):
        log.debug('asynclistener starting runner: %s', self.label)
        listener = AsynchronousRunner(self.listenfunction, *args, **kwargs)
        listener.start()

class AsynchronousRunner(threading.Thread):
    def __init__(self, listenfunction, *args, **kwargs):
        super(AsynchronousRunner, self).__init__()
        self.listenfunction = listenfunction
        self.args = args
        self.kwargs = kwargs
        
    def run(self):
        self.listenfunction(*self.args, **self.kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
from notmm.utils.log import configure_logging

log = logging.getLogger(__name__)
configure_logging(log)

#from utils.wsgilib import HTTPResponse, HTTPRequest

__all__ = ['controllers', 'datastore', 'dbapi', 'utils', 'http', 'release', 'log']            



from livesettings import config_value
from product.models import Product

def recent_products(request):
    """Puts the recently-viewed products in the page variables"""
    recent = request.session.get('RECENTLIST',[])
    maxrecent = config_value('PRODUCT','RECENT_MAX')
        
    products = []
    for slug in recent:
        if len(products) > maxrecent:
            break
            
        try:
            p = Product.objects.get_by_site(slug__exact = slug)
            products.append(p)
        except Product.DoesNotExist:
            pass
    
    return {'recent_products' : products}
    


from satchmo_ext.upsell.models import Upsell, UpsellTranslation
from django.contrib import admin
from django.utils.translation import get_language, ugettext_lazy as _


class UpsellTranslation_Inline(admin.TabularInline):
    model = UpsellTranslation
    extra = 1

class UpsellOptions(admin.ModelAdmin):
    fieldsets = (
        (None, {'fields': ('target', 'goal', 'style', 'notes', 'create_date')}), )
    inlines = [UpsellTranslation_Inline]
    filter_horizontal = ('target',)

admin.site.register(Upsell, UpsellOptions)




from decimal import Decimal
from django.utils.translation import ugettext_lazy as _
from satchmo_store.shop.exceptions import CartAddProhibited
import logging

log = logging.getLogger('upsell.views')

def cart_add_listener(cart=None, product=None, form=None, request=None, **kwargs):
    """Post-processes the form, handling upsell formfields.
    
    fields (potentially) in the form:
        upsell_count: controls how many upsell formblocks to look for.
        for ix in range(0,upsell_count):
            upsell_include_ix: true if this one is to be included
            upsell_qty_ix: quantity
            upsell_slug_ix: product slug        
    """
    log.debug("cart_add_listener heard satchmo_cart_add_complete")
    try:
        rawct = form.get('upsell_count', 0)
        ct = int(rawct)
    except ValueError:
        log.debug("Got bad upsell_count: %s", rawct)
        ct = 0
        
    results = []
    if ct>0:
        for i in range(0,ct):
            results.append(_add_upsell(form, cart, i))
            
    return results
            
def _add_upsell(form, cart, i):
    field_include = "upsell_include_%s" % i
    field_qty = "upsell_qty_%s" % i
    field_slug = "upsell_slug_%s" % i
    slug = ""
    qty = Decimal('0')
    
    if form.get(field_include, "false") == "true":
        slug = form.get(field_slug, "")
        if slug:
            try:
                rawqty = form.get(field_qty, 0)
                if rawqty == "MATCH":
                    qty = Decimal(form['quantity'])
                else:
                    qty = Decimal(rawqty)
            except ValueError:
                log.debug('Bad upsell qty=%d', rawqty)
                qty = Decimal('0')

        if qty > 0:
            from product.models import Product
            try:
                product = Product.objects.get_by_site(slug=slug)
                
                try:
                    cart.add_item(product, number_added=qty)
                    log.info('Added upsell item: %s qty=%d', product.slug, qty)
                    return (True, product)
                                        
                except CartAddProhibited, cap:
                    vetomsg = cap.veto_messages() 
                    msg = _("'%(product)s' couldn't be added to the cart. %(details)s") % {
                        'product' : product.slug, 
                        'detail' : cap.message 
                        }
                    log.debug("Failed to add upsell item: '%s', message= %s", product.slug, msg)
                    return (False, product)
                    
            except Product.DoesNotExist:
                log.debug("Could not find product: %s", slug)
                return (False, None)
                
    return (False, None)



from django.shortcuts import render_to_response
from django.template import RequestContext
from product.models import Product
from satchmo_store.shop import signals
from signals_ahoy.signals import application_search

def search_view(request, template="shop/search.html"):
    """Perform a search based on keywords and categories in the form submission"""
    if request.method=="GET":
        data = request.GET
    else:
        data = request.POST

    keywords = data.get('keywords', '').split(' ')
    category = data.get('category', None)

    keywords = filter(None, keywords)

    results = {}
    
    # this signal will usually call listeners.default_product_search_listener
    application_search.send(Product, request=request, 
        category=category, keywords=keywords, results=results)

    context = RequestContext(request, {
            'results': results,
            'category' : category,
            'keywords' : keywords})
    return render_to_response(template, context_instance=context)



from django.views.decorators.cache import never_cache
from livesettings import config_get_group
from payment.views import confirm, payship
    
def pay_ship_info(request):
    return payship.credit_pay_ship_info(request, config_get_group('PAYMENT_AUTHORIZENET'))
pay_ship_info = never_cache(pay_ship_info)
    
def confirm_info(request):
    return confirm.credit_confirm_info(request, config_get_group('PAYMENT_AUTHORIZENET'))
confirm_info = never_cache(confirm_info)



from django import template
from django.utils.safestring import mark_safe
from l10n.utils import moneyfmt
from satchmo_utils.templatetags import get_filter_args
from shipping.modules.productshipping.models import Carrier

register = template.Library()

def product_shipping(product, args=''):
    if not args:
        raise template.TemplateSyntaxError('product_shipping needs the name of the carrier, as product|productshipping:"carrier"')
    
    try:
        c = Carrier.objects.get(key=args)
    except Carrier.DoesNotExist:
        raise template.TemplateSyntaxError('product_shipping needs the name of a valid carrier, could not find carrier "%s"' % args)
    shipping = c.price(product)
    
    return mark_safe(moneyfmt(shipping))

register.filter(product_shipping)



# -*- coding: UTF-8 -*-
from decimal import Decimal
from django.conf import settings
from django.contrib.sites.models import Site
from django.core import urlresolvers
from django.core.urlresolvers import reverse as url
from django.test import TestCase
from django.test.client import Client
from django.test.client import Client
from l10n.models import *
from livesettings import config_get_group, config_value, config_get
from payment import utils
from product.models import *
from satchmo_store.contact.models import *
from satchmo_store.shop.models import *
from satchmo_utils.dynamic import lookup_template, lookup_url
from urls import make_urlpatterns
import keyedcache

alphabet = 'abcdefghijklmnopqrstuvwxyz'

def make_test_order(country, state, site=None, orderitems=None):
    if not orderitems:
        orderitems = [('dj-rocks-s-b', 5), ('neat-book-hard', 1)]
        
    if not site:
        site = Site.objects.get_current()
        
    c = Contact(first_name="Order", last_name="Tester", 
        role=ContactRole.objects.get(pk='Customer'), email="order@example.com")
    c.save()
    
    if not isinstance(country, Country):
        country = Country.objects.get(iso2_code__iexact = country)
        
    ad = AddressBook(contact=c, description="home",
        street1 = "test", state=state, city="Portland",
        country = country, is_default_shipping=True,
        is_default_billing=True)
    ad.save()
    o = Order(contact=c, shipping_cost=Decimal('10.00'), site = site)
    o.save()
    
    for slug, qty in orderitems:
        p = Product.objects.get(slug=slug)
        price = p.unit_price
        item = OrderItem(order=o, product=p, quantity=qty,
            unit_price=price, line_item_price=price*qty)
        item.save()

    return o

class TestRecurringBilling(TestCase):
    fixtures = ['l10n-data.yaml', 'test_shop.yaml', 'sub_products', 'config']

    def setUp(self):
        self.customer = Contact.objects.create(first_name='Jane', last_name='Doe')
        US = Country.objects.get(iso2_code__iexact="US")
        self.customer.addressbook_set.create(street1='123 Main St', city='New York', state='NY', postal_code='12345', country=US)       
        import datetime
        site = Site.objects.get_current()
        for product in Product.objects.all():
            price, expire_length = self.getTerms(product)
            if price is None:
                continue
            order = Order.objects.create(contact=self.customer, shipping_cost=0, site=site)
            order.orderitem_set.create(
                product=product, 
                quantity=Decimal('1'),
                unit_price=price,
                line_item_price=price,
                expire_date=datetime.datetime.now() + datetime.timedelta(days=expire_length),
                completed=True
            )
            order.recalculate_total()
            order.payments.create(order=order, payment='DUMMY', amount=order.total)
            order.save()
        
    def tearDown(self):
        keyedcache.cache_delete()
        
    def testProductType(self):
        product1 = Product.objects.get(slug='membership-p1')
        product2 = Product.objects.get(slug='membership-p2')
        self.assertEqual(product1.subscriptionproduct.expire_length, 21)
        self.assertEqual(product1.subscriptionproduct.recurring, True)
        self.assertEqual(product1.subscriptionproduct.recurring, True)
        self.assertEqual(product1.price_set.all()[0].price, Decimal('3.95'))
        self.assertEqual(product2.subscriptionproduct.get_trial_terms(0).expire_length, 7)

    def testCheckout(self):
       pass 
        
    def testCronRebill(self):
        for order in OrderItem.objects.all():
            price, expire_length = self.getTerms(order.product)
            if price is None:
                continue
            self.assertEqual(order.expire_date, datetime.date.today() + datetime.timedelta(days=expire_length))
            #set expire date to today for upcoming cron test
            order.expire_date = datetime.date.today()
            order.save()

        order_count = OrderItem.objects.count()
        self.c = Client()
        url = urlresolvers.reverse('satchmo_cron_rebill')
        self.response = self.c.get(url)
        self.assertEqual(self.response.status_code, 200)
        self.assertEqual(self.response.content, 'Authentication Key Required')
        from django.conf import settings
        self.response = self.c.get(url, data={'key': config_value('PAYMENT','CRON_KEY')})
        self.assertEqual(self.response.status_code, 200)
        self.assertEqual(self.response.content, '')
        self.assert_(order_count < OrderItem.objects.count())
        self.assertEqual(order_count, OrderItem.objects.count()/2.0)
        for order in OrderItem.objects.filter(expire_date__gt=datetime.datetime.now()):
            price, expire_length = self.getTerms(order.product, ignore_trial=True)
            if price is None:
                continue
            self.assertEqual(order.expire_date, datetime.date.today() + datetime.timedelta(days=expire_length))
            self.assertEqual(order.order.balance, Decimal('0.00'))
        
    def getTerms(self, object, ignore_trial=False):
        if object.subscriptionproduct.get_trial_terms().count() and ignore_trial is False:
            price = object.subscriptionproduct.get_trial_terms(0).price
            expire_length = object.subscriptionproduct.get_trial_terms(0).expire_length
            return price, expire_length
        elif object.is_subscription or ignore_trial is True:
            price = object.price_set.all()[0].price
            expire_length = object.subscriptionproduct.expire_length
            return price, expire_length
        else:
            return None, None #it's not a recurring object so no test will be done

class TestModulesSettings(TestCase):

    def setUp(self):
        self.dummy = config_get_group('PAYMENT_DUMMY')

    def tearDown(self):
        keyedcache.cache_delete()

    def testGetDummy(self):
        self.assert_(self.dummy != None)
        self.assertEqual(self.dummy.LABEL, 'Payment test module')

    def testLookupTemplateSet(self):
        t = lookup_template(self.dummy, 'test.html')
        self.assertEqual(t, 'test.html')

        # TODO, add these back in when DictValues are added to Comfiguration
        #self.dummy['TEMPLATE_OVERRIDES'] = {'test2.html' : 'foo.html'}
        #t = lookup_template(self.dummy, 'test2.html')
        #self.assertEqual(t, 'foo.html')

    def testLookupURL(self):
        try:
            t = lookup_url(self.dummy, 'test_doesnt_exist')
            self.fail('Should have failed with NoReverseMatch')
        except urlresolvers.NoReverseMatch:
            pass

    def testUrlPatterns(self):
        pats = make_urlpatterns()
        self.assertTrue(len(pats) > 0)

# class TestGenerateCode(TestCase):
# 
#     def testGetCode(self):
#         c = generate_code(alphabet, '^^^^')
# 
#         self.assertEqual(len(c), 4)
# 
#         for ch in c:
#             self.assert_(ch in alphabet)
# 
#     def testGetCode2(self):
#         c = generate_code(alphabet, '^^^^-^^^^')
#         c2 = generate_code(alphabet, '^^^^-^^^^')
#         self.assertNotEqual(c,c2)
# 
#     def testFormat(self):
#         c = generate_code(alphabet, '^-^-^-^')
#         for i in (0,2,4,6):
#             self.assert_(c[i] in alphabet)
#         for i in (1,3,5):
#             self.assertEqual(c[i], '-')

# class TestGenerateCertificateCode(TestCase):
#     def setUp(self):
#         self.charset = config_value('PAYMENT_PAYMENT_GIFTCERTIFICATE', 'CHARSET')
#         self.format = config_value('PAYMENT_PAYMENT_GIFTCERTIFICATE', 'FORMAT')
# 
#     def testGetCode(self):
#         c = generate_certificate_code()
#         self.assertEqual(len(c), len(self.format))
# 
#         chars = [x for x in self.format if not x=='^']
#         chars.extend(self.charset)
#         for ch in c:
#             self.assert_(ch in chars)
# 
# class TestCertCreate(TestCase):
#     def testCreate(self):
#         gc = GiftCertificate(start_balance = '100.00')
#         gc.save()
# 
#         self.assert_(gc.code)
#         self.assertEqual(gc.balance, Decimal('100.00'))
# 
#     def testUse(self):
#         gc = GiftCertificate(start_balance = '100.00')
#         gc.save()
#         bal = gc.use('10.00')
#         self.assertEqual(bal, Decimal('90.00'))
#         self.assertEqual(gc.usages.count(), 1)

class TestMinimumOrder(TestCase):
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'products.yaml', 'test-config.yaml']

    def setUp(self):
        # Every test needs a client
        self.client = Client()
        
    def tearDown(self):
        keyedcache.cache_delete()

    def test_checkout_minimums(self):
        """
        Validate we can add some items to the cart
        """
        min_order = config_get('PAYMENT', 'MINIMUM_ORDER')
        
        #start with no min.
        min_order.update("0.00")
        producturl = urlresolvers.reverse("satchmo_product", kwargs={'product_slug' : 'dj-rocks'})
        response = self.client.get(producturl)
        self.assertContains(response, "Django Rocks shirt", count=2, status_code=200)
        cartadd = urlresolvers.reverse('satchmo_cart_add')
        response = self.client.post(cartadd, { "productname" : "dj-rocks",
                                                      "1" : "L",
                                                      "2" : "BL",
                                                      "quantity" : '2'})
        carturl = urlresolvers.reverse('satchmo_cart')                                              
        self.assertRedirects(response, carturl,
            status_code=302, target_status_code=200)
        response = self.client.get(carturl)
        self.assertContains(response, "Django Rocks shirt (Large/Blue)", count=1, status_code=200)
        response = self.client.get(url('satchmo_checkout-step1'))
        self.assertContains(response, "Billing Information", count=1, status_code=200)

        # now check for min order not met
        min_order.update("100.00")
        response = self.client.get(url('satchmo_checkout-step1'))
        self.assertContains(response, "This store requires a minimum order", count=1, status_code=200)
        
        # add a bunch of shirts, to make the min order
        response = self.client.post(cartadd, { "productname" : "dj-rocks",
                                                      "1" : "L",
                                                      "2" : "BL",
                                                      "quantity" : '10'})
        self.assertRedirects(response, carturl,
            status_code=302, target_status_code=200)
        response = self.client.get(url('satchmo_checkout-step1'))
        self.assertContains(response, "Billing Information", count=1, status_code=200)

class TestPaymentHandling(TestCase):
    fixtures = ['l10n-data.yaml', 'sample-store-data.yaml', 'products.yaml', 'test-config.yaml']

    def setUp(self):
        self.client = Client()
        self.US = Country.objects.get(iso2_code__iexact = "US")
                
    def tearDown(self):
        keyedcache.cache_delete()

    def test_authorize(self):
        """Test making an authorization using DUMMY."""
        order = make_test_order(self.US, '')
        self.assertEqual(order.balance, order.total)
        self.assertEqual(order.total, Decimal('125.00'))
        
        processor = utils.get_processor_by_key('PAYMENT_DUMMY')
        processor.create_pending_payment(order=order, amount=order.total)
        
        self.assertEqual(order.pendingpayments.count(), 1)
        self.assertEqual(order.payments.count(), 1)
        
        pending = order.pendingpayments.all()[0]
        self.assertEqual(pending.amount, order.total)
        
        payment = order.payments.all()[0]
        self.assertEqual(payment.amount, Decimal('0'))
        
        self.assertEqual(pending.capture, payment)
        
        self.assertEqual(order.balance_paid, Decimal('0'))
        self.assertEqual(order.authorized_remaining, Decimal('0'))
        
        processor.prepare_data(order)
        result = processor.authorize_payment()
        self.assertEqual(result.success, True)
        auth = result.payment
        self.assertEqual(type(auth), OrderAuthorization)
                
        self.assertEqual(order.authorized_remaining, Decimal('125.00'))
                
        result = processor.capture_authorized_payment(auth)
        self.assertEqual(result.success, True)
        payment = result.payment
        self.assertEqual(auth.capture, payment)
        order = Order.objects.get(pk=order.id)
        self.assertEqual(order.status, 'New')
        self.assertEqual(order.balance, Decimal('0'))

    def test_authorize_multiple(self):
        """Test making multiple authorization using DUMMY."""
        order = make_test_order(self.US, '')
        self.assertEqual(order.balance, order.total)
        self.assertEqual(order.total, Decimal('125.00'))
        
        processor = utils.get_processor_by_key('PAYMENT_DUMMY')
        processor.create_pending_payment(order=order, amount=Decimal('25.00'))
        
        self.assertEqual(order.pendingpayments.count(), 1)
        self.assertEqual(order.payments.count(), 1)
        
        pending = order.pendingpayments.all()[0]
        
        self.assertEqual(pending.amount, Decimal('25.00'))
        processor.prepare_data(order)
        result = processor.authorize_payment()
        self.assertEqual(result.success, True)
        #self.assertEqual(order.authorized_remaining, Decimal('25.00'))
        #self.assertEqual(order.balance, Decimal('100.00'))
                
        processor.create_pending_payment(order=order, amount=Decimal('100.00'))
        result = processor.authorize_payment()
                
        results = processor.capture_authorized_payments()
        self.assertEqual(len(results), 2)
        
        r1 = results[0]
        r2 = results[1]
        self.assertEqual(r1.success, True)
        self.assertEqual(r2.success, True)
        order = Order.objects.get(pk=order.id)
        self.assertEqual(order.status, 'New')
        self.assertEqual(order.balance, Decimal('0'))

    def test_capture(self):
        """Test making a capture without authorization using DUMMY."""
        order = make_test_order(self.US, '')
        self.assertEqual(order.balance, order.total)
        self.assertEqual(order.total, Decimal('125.00'))
        
        processor = utils.get_processor_by_key('PAYMENT_DUMMY')
        processor.create_pending_payment(order=order, amount=order.total)
        
        processor.prepare_data(order)
        result = processor.capture_payment()
        self.assertEqual(result.success, True)
        pmt1 = result.payment
        self.assertEqual(type(pmt1), OrderPayment)
                
        self.assertEqual(order.authorized_remaining, Decimal('0.00'))
                
        self.assertEqual(result.success, True)
        payment = result.payment
        self.assertEqual(pmt1, payment)
        order = Order.objects.get(pk=order.id)
        self.assertEqual(order.status, 'New')
        self.assertEqual(order.balance, Decimal('0'))



    def test_multiple_pending(self):
        """Test that creating a second pending payment deletes the first one."""
        order = make_test_order(self.US, '')
        self.assertEqual(order.balance, order.total)
        self.assertEqual(order.total, Decimal('125.00'))
        
        processor = utils.get_processor_by_key('PAYMENT_DUMMY')
        pend1 = processor.create_pending_payment(order=order, amount=order.total)
        pend2 = processor.create_pending_payment(order=order, amount=order.total)
        
        self.assertEqual(order.pendingpayments.count(), 1)
        self.assertEqual(order.payments.count(), 1)



import sys

MUTATORS = '__elixir_mutators__'

class ClassMutator(object):
    '''
    DSL-style syntax

    A ``ClassMutator`` object represents a DSL term.
    '''

    def __init__(self, handler):
        '''
        Create a new ClassMutator, using the `handler` callable to process it
        when the time will come.
        '''
        self.handler = handler

    # called when a mutator (eg. "has_field(...)") is parsed
    def __call__(self, *args, **kwargs):
        # self in this case is the "generic" mutator (eg "has_field")

        # jam this mutator into the class's mutator list
        class_locals = sys._getframe(1).f_locals
        mutators = class_locals.setdefault(MUTATORS, [])
        mutators.append((self, args, kwargs))

    def process(self, entity, *args, **kwargs):
        '''
        Process one mutator. This version simply calls the handler callable,
        but another mutator (sub)class could do more processing.
        '''
        self.handler(entity, *args, **kwargs)


#TODO: move this to the super class (to be created here) of EntityMeta
def process_mutators(entity):
    '''
    Apply all mutators of the given entity. That is, loop over all mutators
    in the class's mutator list and process them.
    '''
    # we don't use getattr here to not inherit from the parent mutators
    # inadvertantly if the current entity hasn't defined any mutator.
    mutators = entity.__dict__.get(MUTATORS, [])
    for mutator, args, kwargs in mutators:
        mutator.process(entity, *args, **kwargs)

class Statement(ClassMutator):

    def process(self, entity, *args, **kwargs):
        builder = self.handler(entity, *args, **kwargs)
        entity._descriptor.builders.append(builder)

class PropertyStatement(ClassMutator):

    def process(self, entity, name, *args, **kwargs):
        prop = self.handler(*args, **kwargs)
        prop.attach(entity, name)




# Copyright (C) 2005 by Imendio AB
#               2005 by Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from gazpacho.widgetregistry import widget_registry
from gazpacho.interfaces import BaseLibrary
from gazpacho.loader.custom import adapter_registry

__all__ = ['LibraryLoadError', 'load_library', 'registry_library']

class LibraryLoadError(Exception):
    pass

class LibraryRegistry:
    def __init__(self):
        self._libraries = {}

    def register(self, name, library_class):
        if name in self._libraries:
            raise ValueError(
                "A library called %s is already registered: %s" % name)

        self._libraries[name] = library_class

    def get(self, name):
        if not name in self._libraries:
            raise ValueError("Unknown library: %s" % name)

        return self._libraries[name]

_registry = LibraryRegistry()

def registry_library(name, library_class):
    """
    Register a library, can be used by external parts to be
    able to write bindings for other langauges.
    library_class must be a subclass of BaseLibrary

    @param name:          name of the library loader
    @param library_class: class to be called
    """

    global _registry

    if not isinstance(library_class, BaseLibrary):
        raise TypeError("library_class must be a subclass of BaseLibrary")

    _registry.register(name, library_class)

def load_library(library_type, name, library_name):
    """
    Load a library, named library_type

    @param library_type:
    @param name:
    @param library_name:
    """
    global _registry
    library = _registry.get(library_type)
    return library(name, library_name)

# Register a python library/loader
class PythonLibrary(BaseLibrary):
    def __init__(self, name, library_name):
        BaseLibrary.__init__(self, name, library_name)
        self.name = name

        self.library_name = library_name

        if not library_name:
            library_name = name
        full = 'gazpacho.widgets.' + name
        namespace = locals()
        try:
            module = __import__(full, namespace, namespace)
            self._fullname = full
        except ImportError as e1:
            try:
                module = __import__(library_name, namespace, namespace)
                self._fullname = library_name
            except ImportError as e2:
                raise LibraryLoadError(
                    "ImportError: %s failed to load with error \"%s\""
                    ", while %s failed to load with error \"%s\"" %
                    (full, e1, library_name, e2))
        self._module = module

    def _get_adaptor_class(self, type_name, class_name):
        klass = None

        if class_name:
            longname = self._fullname + '.' + class_name
            parts = longname.split('.')
            modulename = '.'.join(parts[:-1])
            class_name = parts[-1]

            try:
                module = __import__(modulename, {}, {}, modulename)
            except ImportError:
                print("Could not load module: %s" % modulename)
                raise
            except SyntaxError:
                raise

            klass = getattr(module, class_name, None)

        if klass is None:
            try:
                klass = type(widget_registry.get_by_name_closest(type_name))
            except RuntimeError:
                raise LibraryLoadError(
                    'Failed to get ancestor for %s' % type_name)
        return klass

    def get_property_adaptor_class(self, type_name, property_name,
                                   property_type, class_name):
        return self._get_adaptor_class(type_name, class_name)

    def get_widget_adaptor_class(self, type_name, class_name):
        return self._get_adaptor_class(type_name, class_name)

    def get_class(self, class_name):
        return self._namespace.get(class_name)

    def create_widget(self, gtype):
        adapter = adapter_registry.get_adapter(gtype, None)
        return adapter.construct('', gtype, {})

_registry.register('python', PythonLibrary)




"""Pluggable newsletter handling."""

from django import forms
from django.utils.translation import ugettext_lazy as _
from livesettings import config_value
from satchmo_store.accounts.signals import satchmo_registration
from satchmo_store.contact.signals import satchmo_contact_view
from satchmo_utils import load_module
from signals_ahoy.signals import form_initialdata
import logging
import signals

log = logging.getLogger('newsletter')

def get_newsletter_module():
    try:
        modulename = config_value('NEWSLETTER', 'MODULE')
    except AttributeError:
        modulename = 'satchmo_ext.newsletter.ignore'
        
    return load_module(modulename)

def is_subscribed(contact):
    if not contact:
        return False
    return get_newsletter_module().is_subscribed(contact)

def update_subscription(contact, subscribed, attributes={}):
    current = is_subscribed(contact)
    log.debug("Updating subscription status from %s to %s for %s", current, subscribed, contact)
    result = get_newsletter_module().update_contact(contact, subscribed, attributes=attributes)
    signals.newsletter_subscription_updated.send(contact, 
        old_state=current, new_state=subscribed, contact=contact, attributes=attributes)
    return result

def update_subscription_listener(contact=None, subscribed=False, **kwargs):
    if contact:
        update_subscription(contact, subscribed)

def populate_form_initialdata_listener(contact=None, initial = {}, **kwargs):
    if contact:
        current_subscriber = is_subscribed(contact)
    else:
        current_subscriber = False

    initial['newsletter'] = current_subscriber

def view_user_data_listener(contact=None, contact_dict=None, **kwargs):
    module = config_value('NEWSLETTER', 'MODULE')
    if module not in ('', 'satchmo_ext.newsletter.ignore'):
        contact_dict['show_newsletter'] = True
        contact_dict['newsletter'] = is_subscribed(contact)
    else:
        contact_dict['show_newsletter'] = False

satchmo_contact_view.connect(view_user_data_listener, sender=None)
satchmo_registration.connect(update_subscription_listener, sender=None)
form_initialdata.connect(populate_form_initialdata_listener, sender='RegistrationForm')



from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from livesettings import *
from product.config import PRODUCT_GROUP

ENABLE_AKISMET = config_register(
    BooleanValue(PRODUCT_GROUP,
        'AKISMET_ENABLE',
        description= _("Enable Akismet ratings"),
        default=False))
        
AKISMET = config_register(
    StringValue(PRODUCT_GROUP,
        'AKISMET_KEY',
        description= _("Akismet API Key"),
        requires=ENABLE_AKISMET,
        default=""))



'''
Canada Post Shipping Module
v0.1.1
'''
from django.utils.translation import ugettext_lazy as _
from livesettings import *

SHIP_MODULES = config_get('SHIPPING', 'MODULES')
SHIP_MODULES.add_choice(('shipping.modules.canadapost', 'Canada Post'))

SHIPPING_GROUP = ConfigurationGroup('shipping.modules.canadapost',
  _('Canada Post Shipping Settings'),
  requires = SHIP_MODULES,
  requiresvalue='shipping.modules.canadapost',
  ordering = 101
)

config_register_list(

    StringValue(SHIPPING_GROUP,
        'CPCID',
        description=_('Canada Post Merchant ID'),
        help_text=_('The merchant ID assigned by Canada Post'),
        default=u'CPC_DEMO_XML'),

    #http://sellonline.canadapost.ca/DevelopersResources/protocolV3/ProductID.html
    MultipleStringValue(SHIPPING_GROUP,
        'CANADAPOST_SHIPPING_CHOICES',
        description=_("Canada Post shipping choices available to customers."),
        choices = (
            (('1010', 'Domestic - Regular')),
            (('1020', 'Domestic - Expedited')),
            (('1030', 'Domestic - Xpresspost')),            
            (('1040', 'Domestic - Priority Courier')),
            (('2005', 'US - Small Packets Surface')),
            (('2015', 'US - Small Packets Air')),
            (('2020', 'US - Expedited US Business Contract')),
            (('2025', 'US - Expedited US Commercial')),
            (('2030', 'US - Xpress USA')),
            (('2040', 'US - Priority Worldwide USA')),
            (('2050', 'US - Priority Worldwide PAK USA')),
            (('3005', 'Int`l - Small Packets Surface')),
            (('3010', 'Int`l - Surface International')),
            (('3015', 'Int`l - Small Packets Air')),
            (('3020', 'Int`l - Air International')),
            (('3025', 'Int`l - Xpresspost International')),
            (('3040', 'Int`l - Priority Worldwide INTL')),
            (('3050', 'Int`l - Priority Worldwide PAK INTL')),
        ),
        default = ('1010','1020','1030','1040',)),

    StringValue(SHIPPING_GROUP,
        'SHIPPING_CONTAINER',
        description=_("Type of container used to ship product."),
        choices = (
            (('00', 'Unknown')),
            (('01', 'Variable')),
            (('02', 'Rectangular')),
        ),
        default = u"00"),

    BooleanValue(SHIPPING_GROUP,
        'LIVE',
        description=_('Access production Canada Post server'),
        help_text=_('Use this when your store is in production.'),
        default=False),

    StringValue(SHIPPING_GROUP,
        'CONNECTION',
        description=_('Submit to URL'),
        help_text=_('Canada Post Sell Online server to submit live transactions.'),
        default='http://sellonline.canadapost.ca:30000'),

    StringValue(SHIPPING_GROUP,
        'CONNECTION_TEST',
        description=_('Submit to TestURL'),
        help_text=_('Canada Post Sell Online server to submit test transactions.'),
        default='http://sellonline.canadapost.ca:30000'),

    StringValue(SHIPPING_GROUP,
        'TURN_AROUND_TIME',
        description=_('Turn around time'),
        help_text=_('Turn around time in hours. If declared here, this parameter \
                    will overwrite the one defined in the merchant\'s profile'),
        default='24'),

    BooleanValue(SHIPPING_GROUP,
        'VERBOSE_LOG',
        description=_("Verbose logs"),
        help_text=_("Send the entire request and response to the log - for debugging help when setting up Canada Post."),
        default=False)
)



#!/usr/bin/env python
# Copyright (c) Etienne Robillard 2007-2009 <erob@gthcfoundation.org>

from notmm.utils.wsgilib import HTTPRedirectResponse
from django.shortcuts import render_to_response

__all__ = ('handle302', 'handle303', 'handle404', 'handle500')

def handle302(request, **kwargs):
    location = kwargs['location']
    return HTTPRedirectResponse(location=location, status=302)

def handle303(request, **kwargs):
    location = kwargs['location']
    return HTTPRedirectResponse(location=location, status=303)

def handle500(request, **kwargs):
    template_name = '500.mako'
    return render_to_response(request, template_name, status=500)
 
def handle404(request, **kwargs):
    #self.logger.info('Location=%r' % request.path_url)
    #self.logger.info('Client IP=%r' % request.remote_addr)
    template_name = '404.mako'
    return render_to_response(request, template_name, status=404)
    
